from operator import attrgetter

from snowflake.core._internal.utils import normalize_and_unquote_name


def test_iter(stages, temp_stage, temp_stage_case_sensitive):
    stage_names = tuple(map(attrgetter("name"), stages.iter()))
    assert any(
        map(
            lambda e: e in stage_names,
            (
                temp_stage.name.upper(),  # for upper/lower case names
                normalize_and_unquote_name(temp_stage_case_sensitive.name),
            ),
        )
    )


def test_iter_like(stages, temp_stage, temp_stage_case_sensitive):
    stage_names = tuple(map(attrgetter("name"), stages.iter(like="test_stage%")))
    assert any(
        map(
            lambda e: e in stage_names,
            (
                temp_stage.name.upper(),  # for upper/lower case names
                normalize_and_unquote_name(temp_stage_case_sensitive.name),
            ),
        )
    )

    # no result for random
    stage_names = list(stages.iter(like="RANDOM"))
    assert len(stage_names) == 0



import pytest as pytest

from snowflake.core._common import CreateMode
from snowflake.core.procedure import (
    Argument,
    CallArgument,
    CallArgumentList,
    ColumnType,
    JavaFunction,
    JavaScriptFunction,
    Procedure,
    ReturnDataType,
    ReturnTable,
    SQLFunction,
)


@pytest.mark.min_sf_ver("8.38.0")
def test_call_procedures(procedures, cursor, populate_tables_for_procedure_call, maven_snowpark_jar_available):
    del maven_snowpark_jar_available
    javascript_proc_with_args = Procedure(
        name="javascript_proc_with_args",
        arguments=[Argument(name="float1", datatype="FLOAT"), Argument(name="float2", datatype="FLOAT")],
        return_type=ReturnDataType(datatype="FLOAT"),
        language_config=JavaScriptFunction(),
        body="""return FLOAT1""",
    )
    try:
        proc = procedures.create(javascript_proc_with_args, mode=CreateMode.or_replace)
        assert proc.call(
            call_argument_list=CallArgumentList(
                call_arguments=[
                    CallArgument(name="float1", datatype="FLOAT", value=3.14),
                    CallArgument(name="float2", datatype="FLOAT", value=4.13),
                ]
            )
        ) == [{f"{javascript_proc_with_args.name}": 3.14}]
    finally:
        proc.drop()

    sql_proc_table_func = Procedure(
        name="sql_proc_table_func",
        arguments=[Argument(name="id", datatype="VARCHAR")],
        return_type=ReturnTable(
            column_list=[ColumnType(name="id", datatype="NUMBER"), ColumnType(name="price", datatype="NUMBER")]
        ),
        language_config=SQLFunction(),
        body="""DECLARE
                    res RESULTSET DEFAULT (SELECT * FROM invoices WHERE id = :id);
                BEGIN
                    RETURN TABLE(res);
                END;
            """,
    )
    try:
        proc = procedures.create(sql_proc_table_func, mode=CreateMode.or_replace)
        assert proc.call(
            call_argument_list=CallArgumentList(call_arguments=[CallArgument(name="id", datatype="TEXT", value=1)])
        ) == [{"id": 1, "price": 1}, {"id": 1, "price": 2}]

        assert proc.call(
            call_argument_list=CallArgumentList(call_arguments=[CallArgument(name="id", datatype="TEXT", value=2)])
        ) == [{"id": 2, "price": 3}]
    finally:
        proc.drop()

    java_proc_with_args = Procedure(
        name="java_proc_with_args",
        arguments=[
            Argument(name="bin", datatype="BINARY", default="534E4F57"),
            Argument(name="bool", datatype="BOOLEAN", default="FALSE"),
        ],
        return_type=ReturnDataType(name="ret", datatype="VARCHAR"),
        language_config=JavaFunction(
            runtime_version="11", handler="Test.retGeo", packages=["com.snowflake:snowpark:latest"]
        ),
        # we're returning string since the GEOGRAPHY datatype will accept strings
        body="""class Test {
                    public String retGeo(com.snowflake.snowpark.Session session, String binary, boolean bool) {
                        if(binary.equals("534E4F57")) {
                            return binary;
                        } else {
                            return String.valueOf(bool);
                        }
                    }
                }
            """,
    )
    try:
        proc = procedures.create(java_proc_with_args, mode=CreateMode.or_replace)
        assert proc.call(
            call_argument_list=CallArgumentList(
                call_arguments=[
                    CallArgument(name="bin", datatype="BINARY", value="534E4F57"),
                    CallArgument(name="bool", datatype="BOOLEAN", value=True),
                ]
            )
        ) == [{f"{java_proc_with_args.name}": "534E4F57"}]

        assert proc.call(
            call_argument_list=CallArgumentList(
                call_arguments=[
                    CallArgument(name="bin", datatype="BINARY", value="534E4F56"),
                    CallArgument(name="bool", datatype="BOOLEAN", value=True),
                ]
            )
        ) == [{f"{java_proc_with_args.name}": "true"}]
    finally:
        proc.drop()



from enum import Enum


class Privileges(Enum):
    """Enum for Snowflake privileges that can be granted to a role or user."""

    add_search_optimization = "ADD SEARCH OPTIMIZATION"
    all_privileges = "ALL PRIVILEGES"
    apply = "APPLY"
    applybudget = "APPLYBUDGET"
    apply_aggregation_policy = "APPLY AGGREGATION POLICY"
    apply_authentication_policy = "APPLY AUTHENTICATION POLICY"
    apply_masking_policy = "APPLY MASKING POLICY"
    apply_packages_policy = "APPLY PACKAGES POLICY"
    apply_password_policy = "APPLY PASSWORD POLICY"
    apply_projection_policy = "APPLY PROJECTION POLICY"
    apply_row_access_policy = "APPLY ROW ACCESS POLICY"
    apply_session_policy = "APPLY SESSION POLICY"
    apply_tag = "APPLY TAG"
    attach_listing = "ATTACH LISTING"
    attach_policy = "ATTACH POLICY"
    audit = "AUDIT"
    bind_service_endpoint = "BIND SERVICE ENDPOINT"
    create_account = "CREATE ACCOUNT"
    create_aggregation_policy = "CREATE AGGREGATION POLICY"
    create_alert = "CREATE ALERT"
    create_application = "CREATE APPLICATION"
    create_projection_policy = "CREATE PROJECTION POLICY"
    create_application_package = "CREATE APPLICATION PACKAGE"
    create_authentication_policy = "CREATE AUTHENTICATION POLICY"
    create_compute_pool = "CREATE COMPUTE POOL"
    create_database = "CREATE DATABASE"
    create_database_role = "CREATE DATABASE ROLE"
    create_dataset = "CREATE DATASET"
    create_data_exchange_listing = "CREATE DATA EXCHANGE LISTING"
    create_dynamic_table = "CREATE DYNAMIC TABLE"
    create_event_table = "CREATE EVENT TABLE"
    create_external_table = "CREATE EXTERNAL TABLE"
    create_external_volume = "CREATE EXTERNAL VOLUME"
    create_failover_group = "CREATE FAILOVER GROUP"
    create_file_format = "CREATE FILE FORMAT"
    create_function = "CREATE FUNCTION"
    create_git_repository = "CREATE GIT REPOSITORY"
    create_hybrid_table = "CREATE HYBRID TABLE"
    create_iceberg_table = "CREATE ICEBERG TABLE"
    create_image_repository = "CREATE IMAGE REPOSITORY"
    create_integration = "CREATE INTEGRATION"
    create_masking_policy = "CREATE MASKING POLICY"
    create_materialized_view = "CREATE MATERIALIZED VIEW"
    create_model = "CREATE MODEL"
    create_network_policy = "CREATE NETWORK POLICY"
    create_network_rule = "CREATE NETWORK RULE"
    create_notebook = "CREATE NOTEBOOK"
    create_packages_policy = "CREATE PACKAGES POLICY"
    create_password_policy = "CREATE PASSWORD POLICY"
    create_pipe = "CREATE PIPE"
    create_procedure = "CREATE PROCEDURE"
    create_replication_group = "CREATE REPLICATION GROUP"
    create_role = "CREATE ROLE"
    create_row_access_policy = "CREATE ROW ACCESS POLICY"
    create_schema = "CREATE SCHEMA"
    create_sequence = "CREATE SEQUENCE"
    create_secret = "CREATE SECRET"
    create_service = "CREATE SERVICE"
    create_session_policy = "CREATE SESSION POLICY"
    create_share = "CREATE SHARE"
    create_snapshot = "CREATE SNAPSHOT"
    create_snowflake_core_budget = "CREATE SNOWFLAKE.CORE.BUDGET"
    create_snowflake_ml_anomaly_detection = "CREATE SNOWFLAKE.ML.ANOMALY_DETECTION"
    create_snowflake_ml_forecast = "CREATE SNOWFLAKE.ML.FORECAST"
    create_stage = "CREATE STAGE"
    create_stream = "CREATE STREAM"
    create_streamlit = "CREATE STREAMLIT"
    create_table = "CREATE TABLE"
    create_tag = "CREATE TAG"
    create_task = "CREATE TASK"
    create_user = "CREATE USER"
    create_view = "CREATE VIEW"
    create_warehouse = "CREATE WAREHOUSE"
    delete = "DELETE"
    develop = "DEVELOP"
    evolve_schema = "EVOLVE SCHEMA"
    execute_alert = "EXECUTE ALERT"
    execute_data_metric_function = "EXECUTE DATA METRIC FUNCTION"
    execute_managed_alert = "EXECUTE MANAGED ALERT"
    execute_managed_task = "EXECUTE MANAGED TASK"
    execute_task = "EXECUTE TASK"
    failover = "FAILOVER"
    import_share = "IMPORT SHARE"
    imported_privileges = "IMPORTED PRIVILEGES"
    insert = "INSERT"
    install = "INSTALL"
    manage_account_support_cases = "MANAGE ACCOUNT SUPPORT CASES"
    manage_event_sharing = "MANAGE EVENT SHARING"
    manage_grants = "MANAGE GRANTS"
    manage_listing_autofulfillment = "MANAGE LISTING AUTO FULFILLMENT"
    manage_organization_support_cases = "MANAGE ORGANIZATION SUPPORT CASES"
    manage_releases = "MANAGE RELEASES"
    manage_user_support_cases = "MANAGE USER SUPPORT CASES"
    manage_version = "MANAGE VERSION"
    manage_warehouses = "MANAGE WAREHOUSES"
    modify = "MODIFY"
    modify_log_level = "MODIFY LOG LEVEL"
    modify_session_log_level = "MODIFY SESSION LOG LEVEL"
    modify_trace_level = "MODIFY TRACE LEVEL"
    modify_session_trace_level = "MODIFY SESSION TRACE LEVEL"
    monitor = "MONITOR"
    monitor_execution = "MONITOR EXECUTION"
    monitor_security = "MONITOR SECURITY"
    monitor_usage = "MONITOR USAGE"
    operate = "OPERATE"
    override_share_restrictions = "OVERRIDE SHARE RESTRICTIONS"
    purchase_data_exchange_listing = "PURCHASE DATA EXCHANGE LISTING"
    read = "READ"
    references = "REFERENCES"
    reference_usage = "REFERENCE_USAGE"
    replicate = "REPLICATE"
    resolve_all = "RESOLVE ALL"
    select = "SELECT"
    truncate = "TRUNCATE"
    update = "UPDATE"
    use_any_role = "USE ANY ROLE"
    usage = "USAGE"
    write = "WRITE"



"""
Cortex Inference API.

OpenAPI 3.0 specification for the Cortex REST API  # noqa: E501

The version of the OpenAPI document: 0.1.0
Contact: support@snowflake.com
Generated by: https://openapi-generator.tech

Do not edit this file manually.
"""

from __future__ import annotations

import json
import pprint
import re  # noqa: F401

from typing import Any

from pydantic import BaseModel, ConfigDict, StrictStr


class AnthropicThinking(BaseModel):
    """A model object representing the AnthropicThinking resource.

    Constructs an object of type AnthropicThinking with the provided properties.

    Parameters
    __________
    thinking : str
        Thinking tokens generated during the model response.
        This field can also be used in the follow-up requests to continue the conversation.
    signature : str
        The signature field is an opaque field and should not be interpreted or parsed.
        It should only be used for verifying that the thinking blocks were generated by that specific model.
    """

    thinking: StrictStr

    signature: StrictStr

    __properties = ["thinking", "signature"]

    model_config = ConfigDict(
        validate_by_name=True,
        validate_assignment=True,
    )

    def to_str(self) -> str:
        """Returns the string representation of the model using alias."""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias."""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> AnthropicThinking:
        """Create an instance of AnthropicThinking from a JSON string."""
        return cls.from_dict(json.loads(json_str))

    def to_dict(
        self,
        hide_readonly_properties: bool = False,
    ) -> dict[str, Any]:
        """Returns the dictionary representation of the model using alias."""
        exclude_properties = set()

        if hide_readonly_properties:
            exclude_properties.update({})

        _dict = self.model_dump(serialize_as_any=True, by_alias=True, exclude=exclude_properties, exclude_none=True)

        return _dict

    def to_dict_without_readonly_properties(self) -> dict[str, Any]:
        """Return the dictionary representation of the model without readonly properties."""
        return self.to_dict(hide_readonly_properties=True)

    @classmethod
    def from_dict(cls, obj: dict) -> AnthropicThinking:
        """Create an instance of AnthropicThinking from a dict."""
        if obj is None:
            return None

        if type(obj) is not dict:
            return AnthropicThinking.model_validate(obj)

        _obj = AnthropicThinking.model_validate(
            {
                "thinking": obj.get("thinking"),
                "signature": obj.get("signature"),
            }
        )

        return _obj


class AnthropicThinkingModel:
    def __init__(
        self,
        thinking: str,
        signature: str,
        # optional properties
    ):
        """A model object representing the AnthropicThinking resource.

                Constructs an object of type AnthropicThinking with the provided properties.

                Parameters
                __________
                thinking : str
                    Thinking tokens generated during the model response.
        This field can also be used in the follow-up requests to continue the conversation.

                signature : str
                    The signature field is an opaque field and should not be interpreted or parsed.
        It should only be used for verifying that the thinking blocks were generated by that specific model.
        """
        self.thinking = thinking
        self.signature = signature

    __properties = ["thinking", "signature"]

    def __repr__(self) -> str:
        return repr(self._to_model())

    def _to_model(self):
        return AnthropicThinking(
            thinking=self.thinking,
            signature=self.signature,
        )

    @classmethod
    def _from_model(cls, model) -> AnthropicThinkingModel:
        return AnthropicThinkingModel(
            thinking=model.thinking,
            signature=model.signature,
        )

    def to_dict(self):
        """Create a dictionary of the properties from a AnthropicThinking.

        This method constructs a dictionary with the key-value entries corresponding to the properties of the AnthropicThinking object.

        Returns
        _______
        dict
            A dictionary object created using the input model.
        """
        return self._to_model().to_dict()

    @classmethod
    def from_dict(cls, obj: dict) -> AnthropicThinkingModel:
        """Create an instance of AnthropicThinking from a dict.

        This method constructs a AnthropicThinking object from a dictionary with the key-value pairs of its properties.

        Parameters
        ----------
        obj : dict
            A dictionary whose keys and values correspond to the properties of the resource object.

        Returns
        _______
        AnthropicThinking
            A AnthropicThinking object created using the input dictionary; this will fail if the required properties are missing.
        """
        return cls._from_model(AnthropicThinking.from_dict(obj))


AnthropicThinking._model_class = AnthropicThinkingModel



from typing import Any
from unittest import mock
from unittest.mock import MagicMock

import pytest

from snowflake.core import PollingOperation, RESTConnection
from snowflake.core._common import TokenType
from snowflake.core.spark_connect import SparkConnectResource
from snowflake.core.version import __version__ as VERSION

from ...utils import extra_params


BASE_URL = "http://localhost.me:80/api/v2"
API_CLIENT_REQUEST = "snowflake.core._generated.api_client.ApiClient.request"


@pytest.fixture
def spark_connect_resource():
    conn = RESTConnection("localhost.me", 80, token="", token_type=TokenType.SESSION_TOKEN, protocol="http")
    return SparkConnectResource(conn)


@pytest.mark.parametrize(
    "method, fn",
    (
        ("execute-plan", "execute_plan"),
        ("analyze-plan", "analyze_plan"),
        ("config", "config"),
        ("add-artifacts", "add_artifacts"),
        ("artifact-status", "artifact_status"),
        ("reattach-execute", "reattach_execute"),
        ("release-execute", "release_execute"),
    ),
)
def test_spark_connect_methods(spark_connect_resource, method, fn):
    args = (spark_connect_resource.root, "POST", BASE_URL + f"/spark-connect/{method}")
    kwargs = get_params()

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        mocked_request.return_value = mock_http_response()
        getattr(spark_connect_resource, fn)("")
    mocked_request.assert_called_once_with(*args, **kwargs)

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        mocked_request.return_value = mock_http_response()
        op = getattr(spark_connect_resource, fn + "_async")("")
        assert isinstance(op, PollingOperation)
        op.result()
    mocked_request.assert_called_once_with(*args, **kwargs)


def get_params() -> dict[str, Any]:
    return extra_params(
        headers={
            "Accept": "application/json",
            "User-Agent": "python_api/" + VERSION,
            "Content-Type": "application/octet-stream",
        },
        body="",
    )


def mock_http_response() -> MagicMock:
    m = MagicMock()
    m.data = bytearray()
    m.status = 200
    return m



import pytest

from snowflake.core._internal.utils import normalize_and_unquote_name
from snowflake.core.exceptions import ConflictError
from tests.integ.external_volume.utils import (
    get_aws_external_volume_template,
    get_aws_external_volume_template_case_sensitive,
    get_azure_external_volume_template,
    get_gcs_external_volume_template,
)


pytestmark = [pytest.mark.internal_only]


def test_create_external_volume(external_volumes, setup_credentials_fixture):
    try:
        external_volume_to_drop = []
        # happy path aws
        external_volume = get_aws_external_volume_template()
        external_volume_handler = external_volumes.create(external_volume)
        external_volume_to_drop += [external_volume_handler]
        external_volume_object = external_volume_handler.fetch()
        assert external_volume_object.name == external_volume.name.upper()

        # happy path aws case sensitive
        external_volume = get_aws_external_volume_template_case_sensitive()
        external_volume_handler = external_volumes.create(external_volume)
        external_volume_to_drop += [external_volume_handler]
        external_volume_object = external_volume_handler.fetch()
        assert external_volume_object.name == normalize_and_unquote_name(external_volume.name)
        assert external_volume_object.comment == external_volume.comment

        # already existing external volume
        with pytest.raises(ConflictError):
            external_volumes.create(external_volume)

        # already existing external volume with if_exists
        external_volume_handler = external_volumes.create(external_volume, mode="if_not_exists")

        # using or replace
        external_volume.comment = "SNOWAPI_TEST_EXTERNAL_VOLUME_2"
        external_volume_handler = external_volumes.create(external_volume, mode="or_replace")
        external_volume_object = external_volume_handler.fetch()
        assert external_volume_object.comment == external_volume.comment

        # happy path azure
        external_volume = get_azure_external_volume_template()
        external_volume_handler = external_volumes.create(external_volume)
        external_volume_to_drop += [external_volume_handler]
        external_volume_object = external_volume_handler.fetch()
        assert external_volume_object.name == external_volume.name.upper()

        # happy path gcs
        external_volume = get_gcs_external_volume_template()
        external_volume_handler = external_volumes.create(external_volume)
        external_volume_to_drop += [external_volume_handler]
        external_volume_object = external_volume_handler.fetch()
        assert external_volume_object.name == external_volume.name.upper()

    finally:
        for external_volume_handler in external_volume_to_drop:
            external_volume_handler.drop(if_exists=True)



"""
Snowflake Session API.

The Snowflake Session API is a REST API that you can use to query your current session properties.  # noqa: E501

The version of the OpenAPI document: 0.0.1
Contact: support@snowflake.com
Generated by: https://openapi-generator.tech

Do not edit this file manually.
"""

from __future__ import annotations

import json
import pprint
import re  # noqa: F401

from typing import Any, Optional

from pydantic import BaseModel, ConfigDict, Field, StrictStr


class Parameter(BaseModel):
    """A model object representing the Parameter resource.

    Constructs an object of type Parameter with the provided properties.

    Parameters
    __________
    name : str
        Parameter name.
    value : str, optional
        Parameter value.
    default_value : str, optional
        Default parameter value.
    data_type : str, optional
        Data type of the parameter value. Either BOOLEAN, NUMBER, FLOAT, or STRING.
    level : str, optional
        Level at which parameter is defined.
    description : str, optional
        Parameter description.
    """

    name: StrictStr

    value: Optional[StrictStr] = None

    default_value: Optional[StrictStr] = Field(default=None, alias="defaultValue")

    data_type: Optional[StrictStr] = Field(default=None, alias="dataType")

    level: Optional[StrictStr] = None

    description: Optional[StrictStr] = None

    __properties = ["name", "value", "defaultValue", "dataType", "level", "description"]

    model_config = ConfigDict(
        validate_by_name=True,
        validate_assignment=True,
    )

    def to_str(self) -> str:
        """Returns the string representation of the model using alias."""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias."""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Parameter:
        """Create an instance of Parameter from a JSON string."""
        return cls.from_dict(json.loads(json_str))

    def to_dict(
        self,
        hide_readonly_properties: bool = False,
    ) -> dict[str, Any]:
        """Returns the dictionary representation of the model using alias."""
        exclude_properties = set()

        if hide_readonly_properties:
            exclude_properties.update({})

        _dict = self.model_dump(serialize_as_any=True, by_alias=True, exclude=exclude_properties, exclude_none=True)

        return _dict

    def to_dict_without_readonly_properties(self) -> dict[str, Any]:
        """Return the dictionary representation of the model without readonly properties."""
        return self.to_dict(hide_readonly_properties=True)

    @classmethod
    def from_dict(cls, obj: dict) -> Parameter:
        """Create an instance of Parameter from a dict."""
        if obj is None:
            return None

        if type(obj) is not dict:
            return Parameter.model_validate(obj)

        _obj = Parameter.model_validate(
            {
                "name": obj.get("name"),
                "value": obj.get("value"),
                "default_value": obj.get("defaultValue"),
                "data_type": obj.get("dataType"),
                "level": obj.get("level"),
                "description": obj.get("description"),
            }
        )

        return _obj


class ParameterModel:
    def __init__(
        self,
        name: str,
        # optional properties
        value: Optional[str] = None,
        default_value: Optional[str] = None,
        data_type: Optional[str] = None,
        level: Optional[str] = None,
        description: Optional[str] = None,
    ):
        """A model object representing the Parameter resource.

        Constructs an object of type Parameter with the provided properties.

        Parameters
        __________
        name : str
            Parameter name.
        value : str, optional
            Parameter value.
        default_value : str, optional
            Default parameter value.
        data_type : str, optional
            Data type of the parameter value. Either BOOLEAN, NUMBER, FLOAT, or STRING.
        level : str, optional
            Level at which parameter is defined.
        description : str, optional
            Parameter description.
        """
        self.name = name
        self.value = value
        self.default_value = default_value
        self.data_type = data_type
        self.level = level
        self.description = description

    __properties = ["name", "value", "defaultValue", "dataType", "level", "description"]

    def __repr__(self) -> str:
        return repr(self._to_model())

    def _to_model(self):
        return Parameter(
            name=self.name,
            value=self.value,
            default_value=self.default_value,
            data_type=self.data_type,
            level=self.level,
            description=self.description,
        )

    @classmethod
    def _from_model(cls, model) -> ParameterModel:
        return ParameterModel(
            name=model.name,
            value=model.value,
            default_value=model.default_value,
            data_type=model.data_type,
            level=model.level,
            description=model.description,
        )

    def to_dict(self):
        """Create a dictionary of the properties from a Parameter.

        This method constructs a dictionary with the key-value entries corresponding to the properties of the Parameter object.

        Returns
        _______
        dict
            A dictionary object created using the input model.
        """
        return self._to_model().to_dict()

    @classmethod
    def from_dict(cls, obj: dict) -> ParameterModel:
        """Create an instance of Parameter from a dict.

        This method constructs a Parameter object from a dictionary with the key-value pairs of its properties.

        Parameters
        ----------
        obj : dict
            A dictionary whose keys and values correspond to the properties of the resource object.

        Returns
        _______
        Parameter
            A Parameter object created using the input dictionary; this will fail if the required properties are missing.
        """
        return cls._from_model(Parameter.from_dict(obj))


Parameter._model_class = ParameterModel



def assert_basic_grant(grant, check_granted_by=True):
    if check_granted_by:
        assert grant.granted_by is not None
    assert grant.created_on is not None
    assert grant.containing_scope is None
    assert grant.privileges is not None
    assert grant.securable is not None



"""
Cortex Inference API.

OpenAPI 3.0 specification for the Cortex REST API  # noqa: E501

The version of the OpenAPI document: 0.1.0
Contact: support@snowflake.com
Generated by: https://openapi-generator.tech

Do not edit this file manually.
"""

from __future__ import annotations

import json
import pprint
import re  # noqa: F401

from typing import Any, List, Optional, Union

from pydantic import BaseModel, ConfigDict, Field, StrictBool, StrictInt, StrictStr
from typing_extensions import Annotated

from snowflake.core.cortex.inference_service._generated.models.complete_request_anthropic import (
    CompleteRequestAnthropic,
    CompleteRequestAnthropicModel,
)
from snowflake.core.cortex.inference_service._generated.models.complete_request_messages_inner import (
    CompleteRequestMessagesInner,
    CompleteRequestMessagesInnerModel,
)
from snowflake.core.cortex.inference_service._generated.models.complete_request_openai import (
    CompleteRequestOpenai,
    CompleteRequestOpenaiModel,
)
from snowflake.core.cortex.inference_service._generated.models.complete_request_response_format import (
    CompleteRequestResponseFormat,
    CompleteRequestResponseFormatModel,
)
from snowflake.core.cortex.inference_service._generated.models.guardrails_config import (
    GuardrailsConfig,
    GuardrailsConfigModel,
)
from snowflake.core.cortex.inference_service._generated.models.tool import Tool, ToolModel
from snowflake.core.cortex.inference_service._generated.models.tool_choice import ToolChoice, ToolChoiceModel


class CompleteRequest(BaseModel):
    """A model object representing the CompleteRequest resource.

    Constructs an object of type CompleteRequest with the provided properties.

    Parameters
    __________
    model : str
        The model name. See documentation for possible values.
    messages : list[CompleteRequestMessagesInner]

    anthropic : CompleteRequestAnthropic, optional

    openai : CompleteRequestOpenai, optional

    temperature : float, optional
        Temperature controls the amount of randomness used in response generation. A higher temperature corresponds to more randomness.
    top_p : float,  default 1.0
        Threshold probability for nucleus sampling. A higher top-p value increases the diversity of tokens that the model considers, while a lower value results in more predictable output.
    max_tokens : int,  default 4096
        The maximum number of output tokens to produce. The default value is model-dependent.
    max_output_tokens : int, optional
        Deprecated in favor of "max_tokens", which has identical behavior.
    response_format : CompleteRequestResponseFormat, optional

    guardrails : GuardrailsConfig, optional

    tools : list[Tool], optional
        List of tools to be used during tool calling
    tool_choice : ToolChoice, optional

    provisioned_throughput_id : str, optional
        The provisioned throughput ID to be used with the request.
    sf_ml_xp_inflight_prompt_action : str, optional
        Reserved
    sf_ml_xp_inflight_prompt_client_id : str, optional
        Reserved
    sf_ml_xp_inflight_prompt_public_key : str, optional
        Reserved
    stream : bool,  default True
        Reserved
    """

    model: StrictStr

    anthropic: Optional[CompleteRequestAnthropic] = None

    openai: Optional[CompleteRequestOpenai] = None

    messages: Annotated[List[CompleteRequestMessagesInner], Field(min_length=1)]

    temperature: Optional[
        Union[Annotated[float, Field(strict=True, ge=0.0)], Annotated[int, Field(strict=True, ge=0)]]
    ] = None

    top_p: Optional[
        Union[Annotated[float, Field(le=1.0, strict=True, ge=0.0)], Annotated[int, Field(le=1, strict=True, ge=0)]]
    ] = 1.0

    max_tokens: Optional[Annotated[int, Field(strict=True, ge=0)]] = 4096

    max_output_tokens: Optional[StrictInt] = None

    response_format: Optional[CompleteRequestResponseFormat] = None

    guardrails: Optional[GuardrailsConfig] = None

    tools: Optional[List[Tool]] = None

    tool_choice: Optional[ToolChoice] = None

    provisioned_throughput_id: Optional[StrictStr] = None

    sf_ml_xp_inflight_prompt_action: Optional[StrictStr] = Field(default=None, alias="sf-ml-xp-inflight-prompt-action")

    sf_ml_xp_inflight_prompt_client_id: Optional[StrictStr] = Field(
        default=None, alias="sf-ml-xp-inflight-prompt-client-id"
    )

    sf_ml_xp_inflight_prompt_public_key: Optional[StrictStr] = Field(
        default=None, alias="sf-ml-xp-inflight-prompt-public-key"
    )

    stream: Optional[StrictBool] = True

    __properties = [
        "model",
        "anthropic",
        "openai",
        "messages",
        "temperature",
        "top_p",
        "max_tokens",
        "max_output_tokens",
        "response_format",
        "guardrails",
        "tools",
        "tool_choice",
        "provisioned_throughput_id",
        "sf-ml-xp-inflight-prompt-action",
        "sf-ml-xp-inflight-prompt-client-id",
        "sf-ml-xp-inflight-prompt-public-key",
        "stream",
    ]

    model_config = ConfigDict(
        validate_by_name=True,
        validate_assignment=True,
    )

    def to_str(self) -> str:
        """Returns the string representation of the model using alias."""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias."""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> CompleteRequest:
        """Create an instance of CompleteRequest from a JSON string."""
        return cls.from_dict(json.loads(json_str))

    def to_dict(
        self,
        hide_readonly_properties: bool = False,
    ) -> dict[str, Any]:
        """Returns the dictionary representation of the model using alias."""
        exclude_properties = set()

        if hide_readonly_properties:
            exclude_properties.update({})

        _dict = self.model_dump(serialize_as_any=True, by_alias=True, exclude=exclude_properties, exclude_none=True)

        # override the default output from pydantic by calling `to_dict()` of anthropic
        if self.anthropic:
            _dict["anthropic"] = self.anthropic.to_dict()

        # override the default output from pydantic by calling `to_dict()` of openai
        if self.openai:
            _dict["openai"] = self.openai.to_dict()

        # override the default output from pydantic by calling `to_dict()` of each item in messages (list)
        _items = []
        if self.messages:
            for _item in self.messages:
                if _item:
                    _items.append(_item.to_dict())
            _dict["messages"] = _items

        # override the default output from pydantic by calling `to_dict()` of response_format
        if self.response_format:
            _dict["response_format"] = self.response_format.to_dict()

        # override the default output from pydantic by calling `to_dict()` of guardrails
        if self.guardrails:
            _dict["guardrails"] = self.guardrails.to_dict()

        # override the default output from pydantic by calling `to_dict()` of each item in tools (list)
        _items = []
        if self.tools:
            for _item in self.tools:
                if _item:
                    _items.append(_item.to_dict())
            _dict["tools"] = _items

        # override the default output from pydantic by calling `to_dict()` of tool_choice
        if self.tool_choice:
            _dict["tool_choice"] = self.tool_choice.to_dict()

        # set to None if anthropic (nullable) is None
        if self.anthropic is None:
            _dict["anthropic"] = None

        # set to None if openai (nullable) is None
        if self.openai is None:
            _dict["openai"] = None

        # set to None if temperature (nullable) is None
        if self.temperature is None:
            _dict["temperature"] = None

        # set to None if max_output_tokens (nullable) is None
        if self.max_output_tokens is None:
            _dict["max_output_tokens"] = None

        # set to None if response_format (nullable) is None
        if self.response_format is None:
            _dict["response_format"] = None

        # set to None if guardrails (nullable) is None
        if self.guardrails is None:
            _dict["guardrails"] = None

        # set to None if tool_choice (nullable) is None
        if self.tool_choice is None:
            _dict["tool_choice"] = None

        # set to None if provisioned_throughput_id (nullable) is None
        if self.provisioned_throughput_id is None:
            _dict["provisioned_throughput_id"] = None

        # set to None if stream (nullable) is None
        if self.stream is None:
            _dict["stream"] = None

        return _dict

    def to_dict_without_readonly_properties(self) -> dict[str, Any]:
        """Return the dictionary representation of the model without readonly properties."""
        return self.to_dict(hide_readonly_properties=True)

    @classmethod
    def from_dict(cls, obj: dict) -> CompleteRequest:
        """Create an instance of CompleteRequest from a dict."""
        if obj is None:
            return None

        if type(obj) is not dict:
            return CompleteRequest.model_validate(obj)

        _obj = CompleteRequest.model_validate(
            {
                "model": obj.get("model"),
                "anthropic": CompleteRequestAnthropic.from_dict(obj.get("anthropic"))
                if obj.get("anthropic") is not None
                else None,
                "openai": CompleteRequestOpenai.from_dict(obj.get("openai")) if obj.get("openai") is not None else None,
                "messages": [CompleteRequestMessagesInner.from_dict(_item) for _item in obj.get("messages")]
                if obj.get("messages") is not None
                else None,
                "temperature": obj.get("temperature"),
                "top_p": obj.get("top_p") if obj.get("top_p") is not None else 1.0,
                "max_tokens": obj.get("max_tokens") if obj.get("max_tokens") is not None else 4096,
                "max_output_tokens": obj.get("max_output_tokens"),
                "response_format": CompleteRequestResponseFormat.from_dict(obj.get("response_format"))
                if obj.get("response_format") is not None
                else None,
                "guardrails": GuardrailsConfig.from_dict(obj.get("guardrails"))
                if obj.get("guardrails") is not None
                else None,
                "tools": [Tool.from_dict(_item) for _item in obj.get("tools")]
                if obj.get("tools") is not None
                else None,
                "tool_choice": ToolChoice.from_dict(obj.get("tool_choice"))
                if obj.get("tool_choice") is not None
                else None,
                "provisioned_throughput_id": obj.get("provisioned_throughput_id"),
                "sf_ml_xp_inflight_prompt_action": obj.get("sf-ml-xp-inflight-prompt-action"),
                "sf_ml_xp_inflight_prompt_client_id": obj.get("sf-ml-xp-inflight-prompt-client-id"),
                "sf_ml_xp_inflight_prompt_public_key": obj.get("sf-ml-xp-inflight-prompt-public-key"),
                "stream": obj.get("stream") if obj.get("stream") is not None else True,
            }
        )

        return _obj


class CompleteRequestModel:
    def __init__(
        self,
        model: str,
        messages: list[CompleteRequestMessagesInner],
        # optional properties
        anthropic: Optional[CompleteRequestAnthropic] = None,
        openai: Optional[CompleteRequestOpenai] = None,
        temperature: Optional[float] = None,
        top_p: Optional[float] = 1.0,
        max_tokens: Optional[int] = 4096,
        max_output_tokens: Optional[int] = None,
        response_format: Optional[CompleteRequestResponseFormat] = None,
        guardrails: Optional[GuardrailsConfig] = None,
        tools: Optional[list[Tool]] = None,
        tool_choice: Optional[ToolChoice] = None,
        provisioned_throughput_id: Optional[str] = None,
        sf_ml_xp_inflight_prompt_action: Optional[str] = None,
        sf_ml_xp_inflight_prompt_client_id: Optional[str] = None,
        sf_ml_xp_inflight_prompt_public_key: Optional[str] = None,
        stream: Optional[bool] = True,
    ):
        """A model object representing the CompleteRequest resource.

        Constructs an object of type CompleteRequest with the provided properties.

        Parameters
        __________
        model : str
            The model name. See documentation for possible values.
        messages : list[CompleteRequestMessagesInner]

        anthropic : CompleteRequestAnthropic, optional

        openai : CompleteRequestOpenai, optional

        temperature : float, optional
            Temperature controls the amount of randomness used in response generation. A higher temperature corresponds to more randomness.
        top_p : float,  default 1.0
            Threshold probability for nucleus sampling. A higher top-p value increases the diversity of tokens that the model considers, while a lower value results in more predictable output.
        max_tokens : int,  default 4096
            The maximum number of output tokens to produce. The default value is model-dependent.
        max_output_tokens : int, optional
            Deprecated in favor of "max_tokens", which has identical behavior.
        response_format : CompleteRequestResponseFormat, optional

        guardrails : GuardrailsConfig, optional

        tools : list[Tool], optional
            List of tools to be used during tool calling
        tool_choice : ToolChoice, optional

        provisioned_throughput_id : str, optional
            The provisioned throughput ID to be used with the request.
        sf_ml_xp_inflight_prompt_action : str, optional
            Reserved
        sf_ml_xp_inflight_prompt_client_id : str, optional
            Reserved
        sf_ml_xp_inflight_prompt_public_key : str, optional
            Reserved
        stream : bool,  default True
            Reserved
        """
        self.model = model
        self.anthropic = anthropic
        self.openai = openai
        self.messages = messages
        self.temperature = temperature
        self.top_p = top_p
        self.max_tokens = max_tokens
        self.max_output_tokens = max_output_tokens
        self.response_format = response_format
        self.guardrails = guardrails
        self.tools = tools
        self.tool_choice = tool_choice
        self.provisioned_throughput_id = provisioned_throughput_id
        self.sf_ml_xp_inflight_prompt_action = sf_ml_xp_inflight_prompt_action
        self.sf_ml_xp_inflight_prompt_client_id = sf_ml_xp_inflight_prompt_client_id
        self.sf_ml_xp_inflight_prompt_public_key = sf_ml_xp_inflight_prompt_public_key
        self.stream = stream

    __properties = [
        "model",
        "anthropic",
        "openai",
        "messages",
        "temperature",
        "top_p",
        "max_tokens",
        "max_output_tokens",
        "response_format",
        "guardrails",
        "tools",
        "tool_choice",
        "provisioned_throughput_id",
        "sf-ml-xp-inflight-prompt-action",
        "sf-ml-xp-inflight-prompt-client-id",
        "sf-ml-xp-inflight-prompt-public-key",
        "stream",
    ]

    def __repr__(self) -> str:
        return repr(self._to_model())

    def _to_model(self):
        return CompleteRequest(
            model=self.model,
            anthropic=self.anthropic._to_model() if self.anthropic is not None else None,
            openai=self.openai._to_model() if self.openai is not None else None,
            messages=[x._to_model() for x in self.messages] if self.messages is not None else None,
            temperature=self.temperature,
            top_p=self.top_p,
            max_tokens=self.max_tokens,
            max_output_tokens=self.max_output_tokens,
            response_format=self.response_format._to_model() if self.response_format is not None else None,
            guardrails=self.guardrails._to_model() if self.guardrails is not None else None,
            tools=[x._to_model() for x in self.tools] if self.tools is not None else None,
            tool_choice=self.tool_choice._to_model() if self.tool_choice is not None else None,
            provisioned_throughput_id=self.provisioned_throughput_id,
            sf_ml_xp_inflight_prompt_action=self.sf_ml_xp_inflight_prompt_action,
            sf_ml_xp_inflight_prompt_client_id=self.sf_ml_xp_inflight_prompt_client_id,
            sf_ml_xp_inflight_prompt_public_key=self.sf_ml_xp_inflight_prompt_public_key,
            stream=self.stream,
        )

    @classmethod
    def _from_model(cls, model) -> CompleteRequestModel:
        return CompleteRequestModel(
            model=model.model,
            anthropic=CompleteRequestAnthropicModel._from_model(model.anthropic) if model.anthropic else None,
            openai=CompleteRequestOpenaiModel._from_model(model.openai) if model.openai else None,
            messages=[CompleteRequestMessagesInnerModel._from_model(x) for x in model.messages]
            if model.messages
            else None,
            temperature=model.temperature,
            top_p=model.top_p,
            max_tokens=model.max_tokens,
            max_output_tokens=model.max_output_tokens,
            response_format=CompleteRequestResponseFormatModel._from_model(model.response_format)
            if model.response_format
            else None,
            guardrails=GuardrailsConfigModel._from_model(model.guardrails) if model.guardrails else None,
            tools=[ToolModel._from_model(x) for x in model.tools] if model.tools else None,
            tool_choice=ToolChoiceModel._from_model(model.tool_choice) if model.tool_choice else None,
            provisioned_throughput_id=model.provisioned_throughput_id,
            sf_ml_xp_inflight_prompt_action=model.sf_ml_xp_inflight_prompt_action,
            sf_ml_xp_inflight_prompt_client_id=model.sf_ml_xp_inflight_prompt_client_id,
            sf_ml_xp_inflight_prompt_public_key=model.sf_ml_xp_inflight_prompt_public_key,
            stream=model.stream,
        )

    def to_dict(self):
        """Create a dictionary of the properties from a CompleteRequest.

        This method constructs a dictionary with the key-value entries corresponding to the properties of the CompleteRequest object.

        Returns
        _______
        dict
            A dictionary object created using the input model.
        """
        return self._to_model().to_dict()

    @classmethod
    def from_dict(cls, obj: dict) -> CompleteRequestModel:
        """Create an instance of CompleteRequest from a dict.

        This method constructs a CompleteRequest object from a dictionary with the key-value pairs of its properties.

        Parameters
        ----------
        obj : dict
            A dictionary whose keys and values correspond to the properties of the resource object.

        Returns
        _______
        CompleteRequest
            A CompleteRequest object created using the input dictionary; this will fail if the required properties are missing.
        """
        return cls._from_model(CompleteRequest.from_dict(obj))


CompleteRequest._model_class = CompleteRequestModel



"""
Snowflake Function API.

The Snowflake Function API is a REST API that allows caller to create, execute and drop functions in a Snowflake database.  # noqa: E501

The version of the OpenAPI document: 0.0.1
Contact: support@snowflake.com
Generated by: https://openapi-generator.tech

Do not edit this file manually.
"""

from __future__ import annotations

import json
import pprint
import re

from datetime import datetime
from typing import Any, Optional

from pydantic import ConfigDict, Field, StrictStr, field_validator
from typing_extensions import Annotated

from snowflake.core.function._generated.models.function import Function
from snowflake.core.function._generated.models.function_argument import FunctionArgument, FunctionArgumentModel


class ServiceFunction(Function):
    """A model object representing the ServiceFunction resource.

    Constructs an object of type ServiceFunction with the provided properties.

    Parameters
    __________

    name : str
        Specifies the name for the function, must be unique for the schema in which the function is created

    arguments : list[FunctionArgument]

    service : str
        A Snowflake object identifier. If the identifier contains spaces or special characters, the entire string must be enclosed in double quotes. Identifiers enclosed in double quotes are also case-sensitive.
    endpoint : str
        Function's endpoint
    path : str
        Function's path

    returns : str,  default 'TEXT'
        Specifies the type for the function return value.

    max_batch_rows : int, optional
        Specifies the max rows for batch operation.

    created_on : datetime, optional
        Date and time when the function was created.

    signature : str, optional
        Function's arguments.

    language : str, optional
        Function's language.

    body : str, optional
        Function's body.

    service_database : str, optional
        A Snowflake object identifier. If the identifier contains spaces or special characters, the entire string must be enclosed in double quotes. Identifiers enclosed in double quotes are also case-sensitive.
    service_schema : str, optional
        A Snowflake object identifier. If the identifier contains spaces or special characters, the entire string must be enclosed in double quotes. Identifiers enclosed in double quotes are also case-sensitive.
    """

    service: Annotated[str, Field(strict=True)]

    service_database: Optional[Annotated[str, Field(strict=True)]] = None

    service_schema: Optional[Annotated[str, Field(strict=True)]] = None

    endpoint: StrictStr

    path: StrictStr

    __properties = [
        "function_type",
        "name",
        "arguments",
        "returns",
        "max_batch_rows",
        "created_on",
        "signature",
        "language",
        "body",
    ]

    @field_validator("service")
    def service_validate_regular_expression(cls, v):
        if not re.match(r"""^\"([^\"]|\"\")+\"|[a-zA-Z_][a-zA-Z0-9_$]*$""", v):
            raise ValueError(r"""must validate the regular expression /^"([^"]|"")+"|[a-zA-Z_][a-zA-Z0-9_$]*$/""")
        return v

    @field_validator("service_database")
    def service_database_validate_regular_expression(cls, v):
        if v is None:
            return v
        if not re.match(r"""^\"([^\"]|\"\")+\"|[a-zA-Z_][a-zA-Z0-9_$]*$""", v):
            raise ValueError(r"""must validate the regular expression /^"([^"]|"")+"|[a-zA-Z_][a-zA-Z0-9_$]*$/""")
        return v

    @field_validator("service_schema")
    def service_schema_validate_regular_expression(cls, v):
        if v is None:
            return v
        if not re.match(r"""^\"([^\"]|\"\")+\"|[a-zA-Z_][a-zA-Z0-9_$]*$""", v):
            raise ValueError(r"""must validate the regular expression /^"([^"]|"")+"|[a-zA-Z_][a-zA-Z0-9_$]*$/""")
        return v

    model_config = ConfigDict(
        validate_by_name=True,
        validate_assignment=True,
    )

    def to_str(self) -> str:
        """Returns the string representation of the model using alias."""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias."""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> ServiceFunction:
        """Create an instance of ServiceFunction from a JSON string."""
        return cls.from_dict(json.loads(json_str))

    def to_dict(
        self,
        hide_readonly_properties: bool = False,
    ) -> dict[str, Any]:
        """Returns the dictionary representation of the model using alias."""
        exclude_properties = set()

        if hide_readonly_properties:
            exclude_properties.update({})

        _dict = self.model_dump(serialize_as_any=True, by_alias=True, exclude=exclude_properties, exclude_none=True)

        _dict["function_type"] = Function.get_child_model_discriminator_value("ServiceFunction")

        return _dict

    def to_dict_without_readonly_properties(self) -> dict[str, Any]:
        """Return the dictionary representation of the model without readonly properties."""
        return self.to_dict(hide_readonly_properties=True)

    @classmethod
    def from_dict(cls, obj: dict) -> ServiceFunction:
        """Create an instance of ServiceFunction from a dict."""
        if obj is None:
            return None

        if type(obj) is not dict:
            return ServiceFunction.model_validate(obj)

        _obj = ServiceFunction.model_validate(
            {
                "name": obj.get("name"),
                "arguments": [FunctionArgument.from_dict(_item) for _item in obj.get("arguments")]
                if obj.get("arguments") is not None
                else None,
                "returns": obj.get("returns") if obj.get("returns") is not None else "TEXT",
                "max_batch_rows": obj.get("max_batch_rows"),
                "created_on": obj.get("created_on"),
                "signature": obj.get("signature"),
                "language": obj.get("language"),
                "body": obj.get("body"),
                "service": obj.get("service"),
                "service_database": obj.get("service_database"),
                "service_schema": obj.get("service_schema"),
                "endpoint": obj.get("endpoint"),
                "path": obj.get("path"),
            }
        )

        return _obj


class ServiceFunctionModel(Function):
    def __init__(
        self,
        name: str,
        arguments: list[FunctionArgument],
        service: str,
        endpoint: str,
        path: str,
        # optional properties
        returns: Optional[str] = "TEXT",
        max_batch_rows: Optional[int] = None,
        created_on: Optional[datetime] = None,
        signature: Optional[str] = None,
        language: Optional[str] = None,
        body: Optional[str] = None,
        service_database: Optional[str] = None,
        service_schema: Optional[str] = None,
    ):
        """A model object representing the ServiceFunction resource.

        Constructs an object of type ServiceFunction with the provided properties.

        Parameters
        __________

        name : str
            Specifies the name for the function, must be unique for the schema in which the function is created

        arguments : list[FunctionArgument]

        service : str
            A Snowflake object identifier. If the identifier contains spaces or special characters, the entire string must be enclosed in double quotes. Identifiers enclosed in double quotes are also case-sensitive.
        endpoint : str
            Function's endpoint
        path : str
            Function's path

        returns : str,  default 'TEXT'
            Specifies the type for the function return value.

        max_batch_rows : int, optional
            Specifies the max rows for batch operation.

        created_on : datetime, optional
            Date and time when the function was created.

        signature : str, optional
            Function's arguments.

        language : str, optional
            Function's language.

        body : str, optional
            Function's body.

        service_database : str, optional
            A Snowflake object identifier. If the identifier contains spaces or special characters, the entire string must be enclosed in double quotes. Identifiers enclosed in double quotes are also case-sensitive.
        service_schema : str, optional
            A Snowflake object identifier. If the identifier contains spaces or special characters, the entire string must be enclosed in double quotes. Identifiers enclosed in double quotes are also case-sensitive.
        """
        super().__init__(
            name=name,
            arguments=arguments,
            returns=returns,
            max_batch_rows=max_batch_rows,
            created_on=created_on,
            signature=signature,
            language=language,
            body=body,
        )
        self.service = service
        self.service_database = service_database
        self.service_schema = service_schema
        self.endpoint = endpoint
        self.path = path

    __properties = [
        "function_type",
        "name",
        "arguments",
        "returns",
        "max_batch_rows",
        "created_on",
        "signature",
        "language",
        "body",
    ]

    def __repr__(self) -> str:
        return repr(self._to_model())

    def _to_model(self):
        return ServiceFunction(
            name=self.name,
            arguments=[x._to_model() for x in self.arguments] if self.arguments is not None else None,
            returns=self.returns,
            max_batch_rows=self.max_batch_rows,
            created_on=self.created_on,
            signature=self.signature,
            language=self.language,
            body=self.body,
            service=self.service,
            service_database=self.service_database,
            service_schema=self.service_schema,
            endpoint=self.endpoint,
            path=self.path,
        )

    @classmethod
    def _from_model(cls, model) -> ServiceFunctionModel:
        return ServiceFunctionModel(
            name=model.name,
            arguments=[FunctionArgumentModel._from_model(x) for x in model.arguments] if model.arguments else None,
            returns=model.returns,
            max_batch_rows=model.max_batch_rows,
            created_on=model.created_on,
            signature=model.signature,
            language=model.language,
            body=model.body,
            service=model.service,
            service_database=model.service_database,
            service_schema=model.service_schema,
            endpoint=model.endpoint,
            path=model.path,
        )

    def to_dict(self):
        """Create a dictionary of the properties from a ServiceFunction.

        This method constructs a dictionary with the key-value entries corresponding to the properties of the ServiceFunction object.

        Returns
        _______
        dict
            A dictionary object created using the input model.
        """
        return self._to_model().to_dict()

    @classmethod
    def from_dict(cls, obj: dict) -> ServiceFunctionModel:
        """Create an instance of ServiceFunction from a dict.

        This method constructs a ServiceFunction object from a dictionary with the key-value pairs of its properties.

        Parameters
        ----------
        obj : dict
            A dictionary whose keys and values correspond to the properties of the resource object.

        Returns
        _______
        ServiceFunction
            A ServiceFunction object created using the input dictionary; this will fail if the required properties are missing.
        """
        return cls._from_model(ServiceFunction.from_dict(obj))


ServiceFunction._model_class = ServiceFunctionModel



import pytest

from pydantic_core._pydantic_core import ValidationError

from tests.utils import unquote


pytestmark = [pytest.mark.usefixtures("backup_database_schema")]


def test_iter(schemas, temp_schema, temp_schema_case_sensitive):
    schema_names = [s.name for s in schemas.iter()]
    assert temp_schema.name.upper() in schema_names
    assert unquote(temp_schema_case_sensitive.name) in schema_names


def test_iter_like(schemas, temp_schema, temp_schema_case_sensitive):
    schema_names = [s.name for s in schemas.iter(like="test_schema%")]
    assert temp_schema.name.upper() in schema_names
    assert unquote(temp_schema_case_sensitive.name) in schema_names


def test_iter_starts_with(schemas, temp_schema, temp_schema_case_sensitive):
    schema_names = [s.name for s in schemas.iter(starts_with="Test_schema")]
    assert temp_schema.name.upper() not in schema_names
    assert unquote(temp_schema_case_sensitive.name) not in schema_names

    schema_names = [s.name for s in schemas.iter(starts_with="TEST_SCHEMA")]
    assert temp_schema.name.upper() in schema_names
    assert unquote(temp_schema_case_sensitive.name) not in schema_names


# The limit keyword is required for the from keyword to function, limit=10 was chosen arbitrarily
# as it does not affect the test
def test_iter_from_name(schemas, temp_schema, temp_schema_case_sensitive):
    schema_names = [s.name for s in schemas.iter(limit=10, from_name="test_schema")]
    assert temp_schema.name.upper() not in schema_names
    assert unquote(temp_schema_case_sensitive.name) in schema_names


def test_iter_limit(schemas):
    data = list(schemas.iter(limit=10))
    assert len(data) <= 10

    with pytest.raises(ValidationError):
        list(schemas.iter(limit=10001))



from snowflake.core.external_volume import (
    Encryption,
    ExternalVolume,
    StorageLocationAzure,
    StorageLocationGcs,
    StorageLocationS3,
)
from tests.utils import random_string


def get_aws_external_volume_template():
    return ExternalVolume(
        name=random_string(5, "test_create_aws_external_volume_"),
        storage_locations=[
            StorageLocationS3(
                name="abcd-my-s3-us-west-2",
                storage_base_url="s3://MY_EXAMPLE_BUCKET/",
                storage_aws_role_arn="arn:aws:iam::123456789022:role/myrole",
                encryption=Encryption(type="AWS_SSE_KMS", kms_key_id="1234abcd-12ab-34cd-56ef-1234567890ab"),
            ),
            StorageLocationS3(
                name="abcd-my-s3-us-west-3",
                storage_base_url="s3://MY_EXAMPLE_BUCKET_1/",
                storage_aws_role_arn="arn:aws:iam::123456789022:role/myrole",
                encryption=Encryption(type="AWS_SSE_KMS", kms_key_id="1234abcd-12ab-34cd-56ef-1234567890ab"),
            ),
        ],
        comment="SNOWAPI_TEST_EXTERNAL_VOLUME",
    )


def get_aws_external_volume_template_case_sensitive():
    return ExternalVolume(
        name='"' + random_string(5, "test_create_aws_external_volume_") + '"',
        storage_locations=[
            StorageLocationS3(
                name="abcd-my-s3-us-west-2",
                storage_base_url="s3://MY_EXAMPLE_BUCKET/",
                storage_aws_role_arn="arn:aws:iam::123456789022:role/myrole",
                encryption=Encryption(type="AWS_SSE_KMS", kms_key_id="1234abcd-12ab-34cd-56ef-1234567890ab"),
            )
        ],
        comment="SNOWAPI_TEST_EXTERNAL_VOLUME",
    )


def get_azure_external_volume_template():
    return ExternalVolume(
        name=random_string(5, "test_create_azure_external_volume_"),
        storage_locations=[
            StorageLocationAzure(
                name="abcd-my-azure-northeurope",
                storage_base_url="azure://exampleacct.blob.core.windows.net/my_container_northeurope/",
                azure_tenant_id="a123b4c5-1234-123a-a12b-1a23b45678c9",
            )
        ],
        comment="SNOWAPI_TEST_EXTERNAL_VOLUME",
    )


def get_gcs_external_volume_template():
    return ExternalVolume(
        name=random_string(5, "test_create_gcs_external_volume_"),
        storage_locations=[
            StorageLocationGcs(
                name="abcd-my-us-east-1",
                storage_base_url="gcs://mybucket1/path1/",
                encryption=Encryption(type="GCS_SSE_KMS", kms_key_id="1234abcd-12ab-34cd-56ef-1234567890ab"),
            )
        ],
        comment="SNOWAPI_TEST_EXTERNAL_VOLUME",
    )


def assert_basic(external_volume_object, external_volume_expected):
    assert external_volume_object.comment == external_volume_expected.comment
    assert external_volume_object.allow_writes
    assert external_volume_object.created_on is not None
    assert external_volume_object.owner is not None



"""Manages Snowflake Accounts.

Example:
    >>> account_collection = root.accounts
    >>> account = Account(
    ...     name="MY_ACCOUNT",
    ...     admin_name = "admin"
    ...     admin_password = 'TestPassword1'
    ...     first_name = "Jane"
    ...     last_name = "Smith"
    ...     email = 'myemail@myorg.org'
    ...     edition = "enterprise"
    ...     region = "aws_us_west_2"
    ...  )
    >>> account_collection.create(account)

Refer to :class:`snowflake.core.Root` to create the ``root``.
"""

from ._account import Account, AccountCollection, AccountResource


__all__ = ["Account", "AccountCollection", "AccountResource"]



from unittest import mock

import pytest

from snowflake.core import PollingOperation
from snowflake.core.cortex.chat_service import ChatRequest, CortexChatService

from ...utils import BASE_URL, extra_params


API_CLIENT_REQUEST = "snowflake.core._generated.api_client.ApiClient.request"


@pytest.fixture
def chat_service(fake_root):
    return CortexChatService(fake_root)


def test_chat(fake_root, chat_service):
    args = (fake_root, "POST", BASE_URL + "/cortex/chat")
    kwargs = extra_params(body={"query": ""}, _preload_content=False)

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        op = chat_service.chat_async(ChatRequest(query=""))
        assert isinstance(op, PollingOperation)
        op.result()
    mocked_request.assert_called_once_with(*args, **kwargs)



from unittest import mock

import pytest

from snowflake.core import PollingOperation
from snowflake.core.tag import TagValue
from snowflake.core.view import View, ViewColumn, ViewResource
from snowflake.core.view._generated import TagAssignment, TagReference

from ...utils import BASE_URL, extra_params, mock_http_response


API_CLIENT_REQUEST = "snowflake.core._generated.api_client.ApiClient.request"
VIEW = View(name="my_view", columns=[ViewColumn(name="col1")], query="select col1 from my_tab")


@pytest.fixture
def views(schema):
    return schema.views


@pytest.fixture
def view(views):
    return views["my_view"]


def test_create_view(fake_root, views):
    args = (
        fake_root,
        "POST",
        BASE_URL + "/databases/my_db/schemas/my_schema/views",
    )
    kwargs = extra_params(
        query_params=[],
        body={"name": "my_view", "columns": [{"name": "col1"}], "query": "select col1 from my_tab"},
    )

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        view_res = views.create(VIEW)
        assert isinstance(view_res, ViewResource)
        assert view_res.name == "my_view"
    mocked_request.assert_called_once_with(*args, **kwargs)

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        op = views.create_async(VIEW)
        assert isinstance(op, PollingOperation)
        view_res = op.result()
        assert view_res.name == "my_view"
    mocked_request.assert_called_once_with(*args, **kwargs)


def test_iter_view(fake_root, views):
    args = (fake_root, "GET", BASE_URL + "/databases/my_db/schemas/my_schema/views")
    kwargs = extra_params()

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        mocked_request.return_value = mock_http_response()
        views.iter()
    mocked_request.assert_called_once_with(*args, **kwargs)

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        mocked_request.return_value = mock_http_response()
        op = views.iter_async()
        assert isinstance(op, PollingOperation)
        it = op.result()
        assert list(it) == []
    mocked_request.assert_called_once_with(*args, **kwargs)


def test_fetch_view(fake_root, view):
    from snowflake.core.view._generated.models import View as ViewModel
    from snowflake.core.view._generated.models import ViewColumn as ViewColumnModel

    model = ViewModel(name="my_view", columns=[ViewColumnModel(name="col1")], query="select col1 from my_tab")
    args = (fake_root, "GET", BASE_URL + "/databases/my_db/schemas/my_schema/views/my_view")
    kwargs = extra_params()

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        mocked_request.return_value = mock_http_response(model.to_json())
        view.fetch()
    mocked_request.assert_called_once_with(*args, **kwargs)

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        mocked_request.return_value = mock_http_response(model.to_json())
        op = view.fetch_async()
        assert isinstance(op, PollingOperation)
        view = op.result()
        assert view.to_dict() == VIEW.to_dict()
    mocked_request.assert_called_once_with(*args, **kwargs)


def test_drop_view(fake_root, view):
    args = (fake_root, "DELETE", BASE_URL + "/databases/my_db/schemas/my_schema/views/my_view")
    kwargs = extra_params()

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        view.drop()
    mocked_request.assert_called_once_with(*args, **kwargs)

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        op = view.drop_async()
        assert isinstance(op, PollingOperation)
        op.result()
    mocked_request.assert_called_once_with(*args, **kwargs)


def test_set_tags(fake_root, view, tag):
    args = (fake_root, "POST", BASE_URL + "/databases/my_db/schemas/my_schema/views/my_view:set-tags")
    tags = {tag: TagValue(value="value")}
    kwargs = extra_params(
        body=[
            TagAssignment(
                tag_value=v.value, tag_name=k.name, tag_schema=k.schema.name, tag_database=k.database.name
            ).to_dict()
            for k, v in tags.items()
        ]
    )

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        view.set_tags(tags)
    mocked_request.assert_called_once_with(*args, **kwargs)

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        op = view.set_tags_async(tags)
        assert isinstance(op, PollingOperation)
        op.result()
    mocked_request.assert_called_once_with(*args, **kwargs)


def test_unset_tags(fake_root, view, tag):
    args = (fake_root, "POST", BASE_URL + "/databases/my_db/schemas/my_schema/views/my_view:unset-tags")
    tag_resources = {tag}
    kwargs = extra_params(
        body=[
            TagReference(
                tag_name=tag_res.name, tag_schema=tag_res.schema.name, tag_database=tag_res.database.name
            ).to_dict()
            for tag_res in tag_resources
        ]
    )

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        view.unset_tags(tag_resources)
    mocked_request.assert_called_once_with(*args, **kwargs)

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        op = view.unset_tags_async(tag_resources)
        assert isinstance(op, PollingOperation)
        op.result()
    mocked_request.assert_called_once_with(*args, **kwargs)


def test_get_tags(fake_root, view):
    args = (fake_root, "GET", BASE_URL + "/databases/my_db/schemas/my_schema/views/my_view:get-tags")
    kwargs = extra_params()

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        mocked_request.return_value = mock_http_response()
        assert view.get_tags() == {}
    mocked_request.assert_called_once_with(*args, **kwargs)

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        mocked_request.return_value = mock_http_response()
        op = view.get_tags_async()
        assert isinstance(op, PollingOperation)
        assert op.result() == {}
    mocked_request.assert_called_once_with(*args, **kwargs)



"""
Snowflake User Defined Function API.

The Snowflake User Defined Function API is a REST API that you can use to access, update, and perform certain actions on User Defined Function resource in a Snowflake database.  # noqa: E501

The version of the OpenAPI document: 0.0.1
Contact: support@snowflake.com
Generated by: https://openapi-generator.tech

Do not edit this file manually.
"""

from __future__ import annotations

import json
import pprint
import re  # noqa: F401

from typing import Any, ClassVar, Dict, Optional, Union

from pydantic import BaseModel, ConfigDict, StrictBool

import snowflake.core.user_defined_function._generated.models

from snowflake.core.user_defined_function._generated.models import *  # noqa: F403


class FunctionLanguage(BaseModel):
    """A model object representing the FunctionLanguage resource.

    Constructs an object of type FunctionLanguage with the provided properties.

    Parameters
    __________
    called_on_null_input : bool, optional
        Decide if the function/procedure can receive null input
    is_volatile : bool, optional
         Specifies the behavior of the UDF when returning results. This Field is deprecated for Procedure.

                    If true, UDF might return different values for different rows, even for the same input.
                    This field is deprecated for Procedure.

                    If false, UDF assumes that the function, when called with the same inputs, will always return the same result.
                    This guarantee is not checked. Specifying IMMUTABLE for a UDF that returns different values for the same input will result in undefined behavior.
    """

    called_on_null_input: Optional[StrictBool] = None

    is_volatile: Optional[StrictBool] = None

    __properties = ["language", "called_on_null_input", "is_volatile"]

    model_config = ConfigDict(
        validate_by_name=True,
        validate_assignment=True,
    )

    # JSON field name that stores the object type
    __discriminator_property_name: ClassVar[str] = "language"

    # discriminator mappings
    __discriminator_value_class_map: ClassVar[Dict[str, str]] = {
        "JAVA": "JavaFunction",
        "JAVASCRIPT": "JavaScriptFunction",
        "PYTHON": "PythonFunction",
        "SCALA": "ScalaFunction",
        "SQL": "SQLFunction",
        "BaseLanguage": "BaseLanguage",
    }

    @classmethod
    def get_discriminator_value(cls, obj: dict[str, Any]) -> Optional[str]:
        """Returns the discriminator value (object type) of the data."""
        discriminator_value = obj[cls.__discriminator_property_name]
        if discriminator_value:
            return cls.__discriminator_value_class_map.get(discriminator_value)
        else:
            return None

    __discriminator_value_to_type: ClassVar[Dict[str, str]] = {
        "JavaFunction": "JAVA",
        "JavaScriptFunction": "JAVASCRIPT",
        "PythonFunction": "PYTHON",
        "ScalaFunction": "SCALA",
        "SQLFunction": "SQL",
        "BaseLanguage": "BaseLanguage",
    }

    @classmethod
    def get_child_model_discriminator_value(cls, child_model: str) -> str:
        return cls.__discriminator_value_to_type[child_model]

    def to_str(self) -> str:
        """Returns the string representation of the model using alias."""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias."""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(
        cls, json_str: str
    ) -> Union[BaseLanguage, JavaFunction, JavaScriptFunction, PythonFunction, SQLFunction, ScalaFunction]:
        """Create an instance of FunctionLanguage from a JSON string."""
        return cls.from_dict(json.loads(json_str))

    def to_dict(
        self,
        hide_readonly_properties: bool = False,
    ) -> dict[str, Any]:
        """Returns the dictionary representation of the model using alias."""
        exclude_properties = set()

        if hide_readonly_properties:
            exclude_properties.update({})

        _dict = self.model_dump(serialize_as_any=True, by_alias=True, exclude=exclude_properties, exclude_none=True)

        return _dict

    def to_dict_without_readonly_properties(self) -> dict[str, Any]:
        """Return the dictionary representation of the model without readonly properties."""
        return self.to_dict(hide_readonly_properties=True)

    @classmethod
    def from_dict(
        cls, obj: dict
    ) -> Union[BaseLanguage, JavaFunction, JavaScriptFunction, PythonFunction, SQLFunction, ScalaFunction]:
        """Create an instance of FunctionLanguage from a dict."""
        # look up the object type based on discriminator mapping
        object_type = cls.get_discriminator_value(obj)
        if object_type:
            klass = getattr(snowflake.core.user_defined_function._generated.models, object_type)
            return klass.from_dict(obj)
        else:
            raise ValueError(
                "FunctionLanguage failed to lookup discriminator value from "
                + json.dumps(obj)
                + ". Discriminator property name: "
                + cls.__discriminator_property_name
                + ", mapping: "
                + json.dumps(cls.__discriminator_value_class_map)
            )


class FunctionLanguageModel:
    def __init__(
        self,  # optional properties
        called_on_null_input: Optional[bool] = None,
        is_volatile: Optional[bool] = None,
    ):
        """A model object representing the FunctionLanguage resource.

        Constructs an object of type FunctionLanguage with the provided properties.

        Parameters
        __________
        called_on_null_input : bool, optional
            Decide if the function/procedure can receive null input
        is_volatile : bool, optional
             Specifies the behavior of the UDF when returning results. This Field is deprecated for Procedure.

            If true, UDF might return different values for different rows, even for the same input.
            This field is deprecated for Procedure.

            If false, UDF assumes that the function, when called with the same inputs, will always return the same result.
            This guarantee is not checked. Specifying IMMUTABLE for a UDF that returns different values for the same input will result in undefined behavior.
        """
        self.called_on_null_input = called_on_null_input
        self.is_volatile = is_volatile

    __properties = ["language", "called_on_null_input", "is_volatile"]

    def __repr__(self) -> str:
        return repr(self._to_model())

    def _to_model(self):
        return FunctionLanguage(
            called_on_null_input=self.called_on_null_input,
            is_volatile=self.is_volatile,
        )

    @classmethod
    def _from_model(cls, model) -> FunctionLanguageModel:
        return model.__class__._model_class._from_model(model)

    def to_dict(self):
        """Create a dictionary of the properties from a FunctionLanguage.

        This method constructs a dictionary with the key-value entries corresponding to the properties of the FunctionLanguage object.

        Returns
        _______
        dict
            A dictionary object created using the input model.
        """
        return self._to_model().to_dict()

    @classmethod
    def from_dict(
        cls, obj: dict
    ) -> Union[
        BaseLanguageModel,
        JavaFunctionModel,
        JavaScriptFunctionModel,
        PythonFunctionModel,
        SQLFunctionModel,
        ScalaFunctionModel,
    ]:
        """Create an instance of FunctionLanguage from a dict.

        This method constructs a FunctionLanguage object from a dictionary with the key-value pairs of its properties.

        Parameters
        ----------
        obj : dict
            A dictionary whose keys and values correspond to the properties of the resource object.

        Returns
        _______
        FunctionLanguage
            A FunctionLanguage object created using the input dictionary; this will fail if the required properties are missing.
        """
        return cls._from_model(FunctionLanguage.from_dict(obj))


FunctionLanguage._model_class = FunctionLanguageModel



"""Manages Snowpark Container Compute Pools.

Example:
    >>> new_pool_def = ComputePool(
    ...     name="MYCOMPUTEPOOL", instance_family="STANDARD_1", min_nodes=1, max_nodes=1
    ... )
    >>> new_pool = root.compute_pools.create(new_pool_def)
    >>> cp_snapshot = new_pool.fetch()
    >>> cp_data = root.compute_pools.iter(like=”%COMPUTEPOOL”)
    >>> new_pool.resume()
    >>> new_pool.stop_all_services()
    >>> new_pool.suspend()
    >>> new_pool.delete()
    >>> an_existing_pool = root.compute_pools["existing_compute_pool"]
    >>> an_existing_pool.suspend()

Refer to :class:`snowflake.core.Root` to create the ``root``.
"""

from ._compute_pool import ComputePool, ComputePoolCollection, ComputePoolResource


__all__ = ["ComputePool", "ComputePoolCollection", "ComputePoolResource"]



"""
Snowflake Notification Integration API.

The Snowflake Notification Integration API is a REST API that you can use to access, update, and perform certain actions on Notification Integration resource in a Snowflake database.  # noqa: E501

The version of the OpenAPI document: 0.0.1
Contact: support@snowflake.com
Generated by: https://openapi-generator.tech

Do not edit this file manually.
"""

from __future__ import annotations

import json
import pprint
import re

from datetime import datetime
from typing import Any, Optional

from pydantic import BaseModel, ConfigDict, Field, StrictBool, StrictStr, field_validator
from typing_extensions import Annotated

from snowflake.core.notification_integration._generated.models.notification_hook import (
    NotificationHook,
    NotificationHookModel,
)


class NotificationIntegration(BaseModel):
    """A model object representing the NotificationIntegration resource.

    Constructs an object of type NotificationIntegration with the provided properties.

    Parameters
    __________
    name : str
        Name of the notification.
    notification_hook : NotificationHook

    enabled : bool, optional
        Whether the notification integration is enabled.
    comment : str, optional
        Comment for the notification integration.
    created_on : datetime, optional
        Date and time when the notification was created — **Read-only:** *any user-provided value will be ignored.*
    """

    name: Annotated[str, Field(strict=True)]

    enabled: Optional[StrictBool] = None

    comment: Optional[StrictStr] = None

    created_on: Optional[datetime] = None

    notification_hook: NotificationHook

    __properties = ["name", "enabled", "comment", "created_on", "notification_hook"]

    @field_validator("name")
    def name_validate_regular_expression(cls, v):
        if not re.match(r"""^\"([^\"]|\"\")+\"|[a-zA-Z_][a-zA-Z0-9_$]*$""", v):
            raise ValueError(r"""must validate the regular expression /^"([^"]|"")+"|[a-zA-Z_][a-zA-Z0-9_$]*$/""")
        return v

    model_config = ConfigDict(
        validate_by_name=True,
        validate_assignment=True,
    )

    def to_str(self) -> str:
        """Returns the string representation of the model using alias."""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias."""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> NotificationIntegration:
        """Create an instance of NotificationIntegration from a JSON string."""
        return cls.from_dict(json.loads(json_str))

    def to_dict(
        self,
        hide_readonly_properties: bool = False,
    ) -> dict[str, Any]:
        """Returns the dictionary representation of the model using alias."""
        exclude_properties = set()

        if hide_readonly_properties:
            exclude_properties.update(
                {
                    "created_on",
                }
            )

        _dict = self.model_dump(serialize_as_any=True, by_alias=True, exclude=exclude_properties, exclude_none=True)

        # override the default output from pydantic by calling `to_dict()` of notification_hook
        if self.notification_hook:
            _dict["notification_hook"] = self.notification_hook.to_dict()

        return _dict

    def to_dict_without_readonly_properties(self) -> dict[str, Any]:
        """Return the dictionary representation of the model without readonly properties."""
        return self.to_dict(hide_readonly_properties=True)

    @classmethod
    def from_dict(cls, obj: dict) -> NotificationIntegration:
        """Create an instance of NotificationIntegration from a dict."""
        if obj is None:
            return None

        if type(obj) is not dict:
            return NotificationIntegration.model_validate(obj)

        _obj = NotificationIntegration.model_validate(
            {
                "name": obj.get("name"),
                "enabled": obj.get("enabled"),
                "comment": obj.get("comment"),
                "created_on": obj.get("created_on"),
                "notification_hook": NotificationHook.from_dict(obj.get("notification_hook"))
                if obj.get("notification_hook") is not None
                else None,
            }
        )

        return _obj


class NotificationIntegrationModel:
    def __init__(
        self,
        name: str,
        notification_hook: NotificationHook,
        # optional properties
        enabled: Optional[bool] = None,
        comment: Optional[str] = None,
        created_on: Optional[datetime] = None,
    ):
        """A model object representing the NotificationIntegration resource.

        Constructs an object of type NotificationIntegration with the provided properties.

        Parameters
        __________
        name : str
            Name of the notification.
        notification_hook : NotificationHook

        enabled : bool, optional
            Whether the notification integration is enabled.
        comment : str, optional
            Comment for the notification integration.
        created_on : datetime, optional
            Date and time when the notification was created.
        """
        self.name = name
        self.enabled = enabled
        self.comment = comment
        self.created_on = created_on
        self.notification_hook = notification_hook

    __properties = ["name", "enabled", "comment", "created_on", "notification_hook"]

    def __repr__(self) -> str:
        return repr(self._to_model())

    def _to_model(self):
        return NotificationIntegration(
            name=self.name,
            enabled=self.enabled,
            comment=self.comment,
            created_on=self.created_on,
            notification_hook=self.notification_hook._to_model() if self.notification_hook is not None else None,
        )

    @classmethod
    def _from_model(cls, model) -> NotificationIntegrationModel:
        return NotificationIntegrationModel(
            name=model.name,
            enabled=model.enabled,
            comment=model.comment,
            created_on=model.created_on,
            notification_hook=NotificationHookModel._from_model(model.notification_hook)
            if model.notification_hook
            else None,
        )

    def to_dict(self):
        """Create a dictionary of the properties from a NotificationIntegration.

        This method constructs a dictionary with the key-value entries corresponding to the properties of the NotificationIntegration object.

        Returns
        _______
        dict
            A dictionary object created using the input model.
        """
        return self._to_model().to_dict()

    @classmethod
    def from_dict(cls, obj: dict) -> NotificationIntegrationModel:
        """Create an instance of NotificationIntegration from a dict.

        This method constructs a NotificationIntegration object from a dictionary with the key-value pairs of its properties.

        Parameters
        ----------
        obj : dict
            A dictionary whose keys and values correspond to the properties of the resource object.

        Returns
        _______
        NotificationIntegration
            A NotificationIntegration object created using the input dictionary; this will fail if the required properties are missing.
        """
        return cls._from_model(NotificationIntegration.from_dict(obj))


NotificationIntegration._model_class = NotificationIntegrationModel



from typing import TYPE_CHECKING

from snowflake.core import PollingOperation
from snowflake.core._generated.api_client import StoredProcApiClient
from snowflake.core._internal.telemetry import api_telemetry
from snowflake.core._operation import PollingOperations
from snowflake.core.cortex.embed_service._generated.api import CortexEmbedApi
from snowflake.core.cortex.embed_service._generated.models import EmbedRequest, EmbedResponse


if TYPE_CHECKING:
    from snowflake.core import Root


class CortexEmbedService:
    """Represents the collection operations of the Snowflake Cortex Embed Service resource."""

    def __init__(self, root: "Root") -> None:
        self._api = CortexEmbedApi(root=root, resource_class=None, sproc_client=StoredProcApiClient(root=root))

    @api_telemetry
    def embed(self, model: str, text: list[str]) -> EmbedResponse:
        """Perform embed service, similar to snowflake.cortex.Embed.

        Parameters
        __________
        embed_request: EmbedRequest
            The embed request object to be sent to the embed service. Defined in ./_generated/models/embed_request.py.
        """
        return self._api.embed(EmbedRequest.from_dict({"model": model, "text": text}), async_req=False)

    @api_telemetry
    def embed_async(self, model: str, text: list[str]) -> PollingOperation[EmbedResponse]:
        """An asynchronous version of :func:`embed`.

        Refer to :class:`~snowflake.core.PollingOperation` for more information on asynchronous execution and
        the return type.
        """  # noqa: D401
        future = self._api.embed(EmbedRequest.from_dict({"model": model, "text": text}), async_req=True)
        return PollingOperations.identity(future)



#
# Copyright (c) 2012-2023 Snowflake Computing Inc. All rights reserved.
#


import pytest

from urllib3 import HTTPResponse

from snowflake.core._root import Root
from snowflake.core.cortex.chat_service._generated.models import ChatRequest
from snowflake.core.rest import SSEClient
from snowflake.core.table import Table, TableColumn


TEST_SERVICE_NAME = "SNOWPY_TEST_SERVICE"
TEST_TABLE_NAME = "SNOWPY_TEST_TABLE"

API_NOT_ENABLED = "Cortex Search API is not enabled"

pytestmark = [pytest.mark.skip_gov]


@pytest.fixture(autouse=True)
def setup_cortex_search_service(connection, warehouse, tables, backup_warehouse_fixture):
    del backup_warehouse_fixture
    # Base Table
    tables.create(
        Table(
            name=TEST_TABLE_NAME,
            columns=[
                TableColumn(name="search_col", datatype="varchar"),
                TableColumn(name="filter1", datatype="varchar"),
            ],
        ),
        mode="ifnotexists",
    )

    with connection.cursor() as cursor:
        cursor.execute(f"use warehouse {warehouse.name}").fetchone()

        rows = ",".join(
            [
                "('Citi reimagines financial transactions in the data cloud', 'filter_value_1')",
                "('ADP works in the data cloud. This is another mention of data cloud.', 'filter_value_2')",
                "('Lorem ipsum dolor sit amet', 'filter_value_1')",
                "('Cortex Search is a search solution for any problem', 'filter_value_2')",
            ]
        )
        cursor.execute(f"INSERT INTO {TEST_TABLE_NAME} VALUES {rows}")

        # Cortex Search Service
        cursor.execute(
            f"CREATE OR REPLACE CORTEX SEARCH SERVICE {TEST_SERVICE_NAME} "
            f"ON search_col TARGET_LAG='1 minute' WAREHOUSE={warehouse.name} "
            f"AS SELECT search_col, filter1 FROM {TEST_TABLE_NAME}"
        )

        try:
            yield
        finally:
            cursor.execute(f"DROP CORTEX SEARCH SERVICE {TEST_SERVICE_NAME}")
            cursor.execute(f"DROP TABLE {TEST_TABLE_NAME}")


@pytest.fixture(autouse=True)
def precheck_cortex_search_enabled(cortex_search_services, setup_cortex_search_service):
    del setup_cortex_search_service
    try:
        cortex_search_services[TEST_SERVICE_NAME].search("hi", ["search_col", "filter1"], limit=5)
    except Exception as err:
        if API_NOT_ENABLED in err.body:
            pytest.xfail(API_NOT_ENABLED)
        raise


# Test chat through CortexChatService
# this is just to test the integrity of the python -> cortex chat service.
# We would not try to match the exact response as the response might change over time.
@pytest.mark.flaky
def test_chat_collection(root: Root, database, schema):
    resp = root.cortex_chat_service.chat(
        ChatRequest.from_dict(
            {"query": "data cloud", "search_services": [{"name": f"{database.name}.{schema.name}.{TEST_SERVICE_NAME}"}]}
        )
    )

    for e in resp.events():
        # check if the event and data are not None
        assert e.event is not None
        assert e.data is not None


# Test client parsing of chat service response through XP endpoint
@pytest.mark.flaky
def test_chat_collection_xp(root: Root):
    body_str = (
        "["
        '{"event":"thread.message.delta","data":{"id":"msg_001","object":"thread.message.delta",'
        '"delta":{"content":[{"index":0,"type":"text",'
        '"text":{"value":"Citi reimagines financial transactions in the data cloud 【†2†】."}}]}}},'
        '{"event":"thread.message.delta","data":{"id":"msg_001","object":"thread.message.delta",'
        '"delta":{"content":[{"index":1,"type":"citation",'
        '"citation":{"index":2,"chunk":"Citi reimagines financial transactions in the data cloud",'
        '"ranges":[{"start":41,"end":102}],"title":"","id":""}}]}}}'
        "]"
    )

    response = HTTPResponse(
        body=body_str.encode("utf-8"), status=200, headers={"Content-Type": "application/json"}, preload_content=False
    )
    res = SSEClient(response)

    exp_text = "Citi reimagines financial transactions in the data cloud 【†2†】."
    exp_citation_1 = "Citi reimagines financial transactions in the data cloud"
    event_num = 0
    for e in res.events():
        if event_num == 0:
            assert e.event == "thread.message.delta"
            assert e.data["id"] == "msg_001"
            assert e.data["object"] == "thread.message.delta"
            assert e.data["delta"]["content"][0]["index"] == 0
            assert e.data["delta"]["content"][0]["type"] == "text"
            assert e.data["delta"]["content"][0]["text"]["value"] == exp_text
        if event_num == 1:
            assert e.event == "thread.message.delta"
            assert e.data["id"] == "msg_001"
            assert e.data["object"] == "thread.message.delta"
            assert e.data["delta"]["content"][0]["index"] == 1
            assert e.data["delta"]["content"][0]["type"] == "citation"
            assert e.data["delta"]["content"][0]["citation"]["index"] == 2
            assert e.data["delta"]["content"][0]["citation"]["chunk"] == exp_citation_1
            assert e.data["delta"]["content"][0]["citation"]["ranges"][0]["start"] == 41
            assert e.data["delta"]["content"][0]["citation"]["ranges"][0]["end"] == 102
        event_num += 1

    assert event_num == 2



"""
Snowflake Procedure API.

The Snowflake Procedure API is a REST API that you can use to access, update, and perform certain actions on Procedure resource in a Snowflake database.  # noqa: E501

The version of the OpenAPI document: 0.0.1
Contact: support@snowflake.com
Generated by: https://openapi-generator.tech

Do not edit this file manually.
"""

from __future__ import annotations

import json
import pprint
import re  # noqa: F401

from typing import Any, List, Optional

from pydantic import ConfigDict

from snowflake.core.procedure._generated.models.column_type import ColumnType, ColumnTypeModel
from snowflake.core.procedure._generated.models.return_type import ReturnType


class ReturnTable(ReturnType):
    """A model object representing the ReturnTable resource.

    Constructs an object of type ReturnTable with the provided properties.

    Parameters
    __________
    column_list : list[ColumnType], optional
        List of table columns to return
    """

    column_list: Optional[List[ColumnType]] = None

    __properties = ["type"]

    model_config = ConfigDict(
        validate_by_name=True,
        validate_assignment=True,
    )

    def to_str(self) -> str:
        """Returns the string representation of the model using alias."""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias."""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> ReturnTable:
        """Create an instance of ReturnTable from a JSON string."""
        return cls.from_dict(json.loads(json_str))

    def to_dict(
        self,
        hide_readonly_properties: bool = False,
    ) -> dict[str, Any]:
        """Returns the dictionary representation of the model using alias."""
        exclude_properties = set()

        if hide_readonly_properties:
            exclude_properties.update({})

        _dict = self.model_dump(serialize_as_any=True, by_alias=True, exclude=exclude_properties, exclude_none=True)

        # override the default output from pydantic by calling `to_dict()` of each item in column_list (list)
        _items = []
        if self.column_list:
            for _item in self.column_list:
                if _item:
                    _items.append(_item.to_dict())
            _dict["column_list"] = _items

        _dict["type"] = ReturnType.get_child_model_discriminator_value("ReturnTable")

        return _dict

    def to_dict_without_readonly_properties(self) -> dict[str, Any]:
        """Return the dictionary representation of the model without readonly properties."""
        return self.to_dict(hide_readonly_properties=True)

    @classmethod
    def from_dict(cls, obj: dict) -> ReturnTable:
        """Create an instance of ReturnTable from a dict."""
        if obj is None:
            return None

        if type(obj) is not dict:
            return ReturnTable.model_validate(obj)

        _obj = ReturnTable.model_validate(
            {
                "column_list": [ColumnType.from_dict(_item) for _item in obj.get("column_list")]
                if obj.get("column_list") is not None
                else None,
            }
        )

        return _obj


class ReturnTableModel(ReturnType):
    def __init__(
        self,  # optional properties
        column_list: Optional[list[ColumnType]] = None,
    ):
        """A model object representing the ReturnTable resource.

        Constructs an object of type ReturnTable with the provided properties.

        Parameters
        __________
        column_list : list[ColumnType], optional
            List of table columns to return
        """
        super().__init__()
        self.column_list = column_list

    __properties = ["type"]

    def __repr__(self) -> str:
        return repr(self._to_model())

    def _to_model(self):
        return ReturnTable(
            column_list=[x._to_model() for x in self.column_list] if self.column_list is not None else None,
        )

    @classmethod
    def _from_model(cls, model) -> ReturnTableModel:
        return ReturnTableModel(
            column_list=[ColumnTypeModel._from_model(x) for x in model.column_list] if model.column_list else None,
        )

    def to_dict(self):
        """Create a dictionary of the properties from a ReturnTable.

        This method constructs a dictionary with the key-value entries corresponding to the properties of the ReturnTable object.

        Returns
        _______
        dict
            A dictionary object created using the input model.
        """
        return self._to_model().to_dict()

    @classmethod
    def from_dict(cls, obj: dict) -> ReturnTableModel:
        """Create an instance of ReturnTable from a dict.

        This method constructs a ReturnTable object from a dictionary with the key-value pairs of its properties.

        Parameters
        ----------
        obj : dict
            A dictionary whose keys and values correspond to the properties of the resource object.

        Returns
        _______
        ReturnTable
            A ReturnTable object created using the input dictionary; this will fail if the required properties are missing.
        """
        return cls._from_model(ReturnTable.from_dict(obj))


ReturnTable._model_class = ReturnTableModel



"""Manages Snowflake user roles.

Example:
    >>> role_name = "test_role"
    >>> test_role = Role(name=role_name, comment="test_comment")
    >>> created_role = roles.create(test_role)
    >>> roles[role_name].delete()
"""

from ._role import ContainingScope, Grant, Role, RoleCollection, RoleResource, Securable


__all__ = ["Role", "RoleCollection", "RoleResource", "Securable", "ContainingScope", "Grant"]



from contextlib import suppress

import pytest

from snowflake.core.event_table import EventTable
from snowflake.core.exceptions import NotFoundError
from tests.utils import random_string

from ...utils import ensure_snowflake_version


@pytest.fixture(scope="session")
def event_tables_extended(event_tables, snowflake_version):
    ensure_snowflake_version(snowflake_version, "8.35.0")

    name_list = []
    for _ in range(5):
        name_list.append(random_string(10, "test_event_table_iter_a_"))
    for _ in range(7):
        name_list.append(random_string(10, "test_event_table_iter_b_"))
    for _ in range(3):
        name_list.append(random_string(10, "test_event_table_iter_c_"))

    try:
        for event_table_name in name_list:
            event_tables.create(EventTable(name=event_table_name))

        yield event_tables
    finally:
        for event_table_name in name_list:
            with suppress(NotFoundError):
                event_tables[event_table_name].drop(if_exists=True)


def test_iter_raw(event_tables_extended):
    assert len(list(event_tables_extended.iter())) >= 15


def test_iter_like(event_tables_extended):
    assert len(list(event_tables_extended.iter(like="test_event_table_iter_"))) == 0
    assert len(list(event_tables_extended.iter(like="test_event_table_iter_a_%%"))) == 5
    assert len(list(event_tables_extended.iter(like="test_event_table_iter_b_%%"))) == 7
    assert len(list(event_tables_extended.iter(like="test_event_table_iter_c_%%"))) == 3


def test_iter_show_limit(event_tables_extended):
    assert len(list(event_tables_extended.iter(like="test_event_table_iter_a_%%"))) == 5

    for event_table in event_tables_extended.iter(like="test_event_table_iter_a_%%", show_limit=10):
        assert event_table.name.lower().startswith("test_event_table")
    assert len(list(event_tables_extended.iter(like="test_event_table_iter_a_%%", show_limit=2))) == 2
    assert len(list(event_tables_extended.iter(show_limit=2))) == 2


def test_iter_starts_with(event_tables_extended):
    assert len(list(event_tables_extended.iter(starts_with="test_event_table_iter_a_".upper()))) == 5


def test_iter_from_name(event_tables_extended):
    assert len(list(event_tables_extended.iter(from_name="test_event_table_iter_b_"))) >= 10



from typing import Optional

import pytest

from snowflake.core.managed_account import ManagedAccount, ManagedAccountCollection
from tests.integ.utils import backup_role
from tests.utils import random_string


def assert_managed_account_existence(temp_customer_account_managed_accounts, managed_account_name, should_exist=True):
    managed_account_names = [managed_account.name for managed_account in temp_customer_account_managed_accounts.iter()]
    if should_exist:
        assert managed_account_name.upper() in managed_account_names
    else:
        assert managed_account_name.upper() not in managed_account_names


def get_managed_account_template():
    return ManagedAccount(
        name=random_string(5, "test_create_managed_account_"),
        admin_name="user1",
        admin_password="Sdfed43da!444444",
        account_type="READER",
        comment="This is my managed account@$W%*#$()%",
    )


def create_managed_account(
    temp_customer_account_managed_accounts: ManagedAccountCollection,
    sf_cursor,
    temp_customer_account_cursor,
    managed_account: Optional[ManagedAccount] = None,
    exception: Optional[Exception] = None,
) -> ManagedAccount:
    if managed_account is None:
        managed_account = get_managed_account_template()
    blank_account_name = random_string(5, "test_create_blank_account_")
    with backup_role(sf_cursor):
        sf_cursor.execute("use role accountadmin;")
        sf_cursor.execute("alter session set QA_MODE=true;")
        sf_cursor.execute(f"CREATE ACCOUNT {blank_account_name} server_type=standard type=blank;")

        with backup_role(temp_customer_account_cursor):
            temp_customer_account_cursor.execute("use role accountadmin;")
            if exception is not None:
                with pytest.raises(exception):
                    temp_customer_account_managed_accounts.create(managed_account)
            else:
                temp_customer_account_managed_accounts.create(managed_account)
        sf_cursor.execute("alter session unset QA_MODE;")

    if exception is None:
        assert_managed_account_existence(temp_customer_account_managed_accounts, managed_account.name.upper())
    return managed_account


def cleanup_managed_accounts_completely(temp_customer_account_cursor):
    with backup_role(temp_customer_account_cursor):
        temp_customer_account_cursor.execute("use role accountadmin;")
        managed_accounts_to_drop = temp_customer_account_cursor.execute(
            "SHOW MANAGED ACCOUNTS like 'test_create_managed_account_%';"
        ).fetchall()
        for managed_account_to_drop in managed_accounts_to_drop:
            temp_customer_account_cursor.execute(f"DROP MANAGED ACCOUNT IF EXISTS {managed_account_to_drop[0]};")



from typing import Optional

import pytest

from snowflake.core.streamlit import Streamlit, StreamlitCollection
from snowflake.core.tag import TagResource, TagValue

from ..base_tag_tests import BaseTagTests
from ..utils import random_string


class TestStreamlitTags(BaseTagTests):
    @pytest.fixture(autouse=True)
    def setup(self, streamlits: StreamlitCollection, streamlit_stage_with_file, warehouse, streamlit_main_file):
        self.streamlits = streamlits
        name = random_string(6, "streamlit_for_tag_tests_")
        streamlit = Streamlit(
            name=name,
            query_warehouse=warehouse.name,
            source_location=f"@{streamlit_stage_with_file.name}",
            main_file=streamlit_main_file,
        )
        self.streamlit_res = streamlits.create(streamlit)
        try:
            yield
        finally:
            self.streamlit_res.drop(if_exists=True)

    @property
    def resource_level_name(self) -> str:
        return "STREAMLIT"

    def set_tags(
        self, tags: dict[TagResource, TagValue], if_exists: Optional[bool] = False, resource_name: Optional[str] = None
    ):
        self.streamlits[resource_name or self.streamlit_res.name].set_tags(tags, if_exists)

    def unset_tags(
        self, tag_resources: set[TagResource], if_exists: Optional[bool] = False, resource_name: Optional[str] = None
    ):
        self.streamlits[resource_name or self.streamlit_res.name].unset_tags(tag_resources, if_exists)

    def get_tags(
        self, with_lineage: Optional[bool] = False, resource_name: Optional[str] = None
    ) -> dict[TagResource, TagValue]:
        return self.streamlits[resource_name or self.streamlit_res.name].get_tags(with_lineage)



import pytest

from snowflake.core.role import Role
from tests.utils import random_string


@pytest.mark.use_accountadmin
def test_iter(roles, session):
    role_name_1 = random_string(5, "test_role_1")
    role_name_2 = random_string(5, "test_role_2")
    try:
        test_role = Role(name=role_name_1, comment="test role 1")
        test_role_1 = roles.create(test_role)
        test_role = Role(name=role_name_2, comment="test role 2")
        test_role_2 = roles.create(test_role)

        test_roles = [role.name for role in roles.iter()]

        assert test_role_1.name.upper() in test_roles
        assert test_role_2.name.upper() in test_roles

        # checking the fields of the role
        test_role_random = [role for role in roles.iter()][0]
        assert test_role_random.created_on is not None
        assert test_role_random.assigned_to_users is not None
        assert test_role_random.granted_to_roles is not None
        assert test_role_random.granted_roles is not None
        assert test_role_random.is_default is not None
        assert test_role_random.is_inherited is not None
        assert test_role_random.is_current is not None
        assert test_role_random.comment is not None

        test_roles = [role.name for role in roles.iter(like="TEST_ROLE_%")]

        assert test_role_1.name.upper() in test_roles
        assert test_role_2.name.upper() in test_roles

        test_roles = [role.name for role in roles.iter(like="TEST_ROLE_1%")]

        assert test_role_1.name.upper() in test_roles
        assert test_role_2.name.upper() not in test_roles

        test_roles = roles.iter(limit=1)
        assert len(list(test_roles)) == 1

        # show roles starts_with has a bug and isn't filtering the output as expected.
        # test_roles = roles.iter(starts_with="TEST_ROLE")
        # assert len(list(test_roles)) == 1

    finally:
        roles[test_role_1.name].drop(if_exists=True)
        roles[test_role_2.name].drop(if_exists=True)



import copy

import pytest

from snowflake.core.exceptions import NotFoundError
from tests.integ.utils import random_string

from .conftest import test_sequence_template


def test_rename(sequences):
    original_name = random_string(10, "test_sequence_rename_original_")
    new_name = random_string(10, "test_sequence_rename_")

    sequence = copy.deepcopy(test_sequence_template)
    sequence.name = original_name
    sequence_handle = sequences.create(sequence)

    try:
        fetched_sequence = sequence_handle.fetch()
        assert fetched_sequence.name == original_name.upper()

        sequence_handle.rename(new_name)

        fetched_sequence = sequence_handle.fetch()
        assert fetched_sequence.name == new_name.upper()
        assert fetched_sequence.schema_name == sequences.schema.name.upper()
        assert fetched_sequence.database_name == sequences.database.name.upper()

        with pytest.raises(NotFoundError):
            sequences[original_name].fetch()

    finally:
        sequence_handle.drop(if_exists=True)


def test_rename_nonexistent_sequence(sequences):
    nonexistent_name = random_string(10, "test_sequence_rename_nonexistest_")
    new_name = random_string(10, "test_sequence_rename_")

    with pytest.raises(NotFoundError):
        sequences[nonexistent_name].rename(new_name)

    sequences[nonexistent_name].rename(new_name, if_exists=True)


def test_rename_across_schema(sequences, temp_schema):
    original_name = random_string(10, "test_sequence_rename_cross_schema_original_")
    new_name = random_string(10, "test_sequence_rename_cross_schema_")

    sequence = copy.deepcopy(test_sequence_template)
    sequence.name = original_name
    sequence_handle = sequences.create(sequence)

    try:
        sequence_handle.rename(new_name, target_schema=temp_schema.name)

        fetched_sequence = sequence_handle.fetch()
        assert fetched_sequence.name == new_name.upper()
        assert fetched_sequence.schema_name == temp_schema.name.upper()
        assert fetched_sequence.database_name == sequences.database.name.upper()

        with pytest.raises(NotFoundError):
            sequences[original_name].fetch()

    finally:
        sequence_handle.drop(if_exists=True)


def test_rename_across_database(sequences, temp_db):
    original_name = random_string(10, "test_sequence_rename_cross_db_original_")
    new_name = random_string(10, "test_sequence_rename_cross_db_")

    sequence = copy.deepcopy(test_sequence_template)
    sequence.name = original_name
    sequence_handle = sequences.create(sequence)

    try:
        sequence_handle.rename(new_name, target_database=temp_db.name, target_schema="PUBLIC")

        fetched_sequence = sequence_handle.fetch()
        assert fetched_sequence.name == new_name.upper()
        assert fetched_sequence.schema_name == "PUBLIC"
        assert fetched_sequence.database_name == temp_db.name.upper()

        with pytest.raises(NotFoundError):
            sequences[original_name].fetch()

    finally:
        sequence_handle.drop(if_exists=True)



import pytest

from snowflake.core.account import Account
from snowflake.core.account._generated.models import Account as BaseAccount
from snowflake.core.compute_pool import ComputePool
from snowflake.core.compute_pool._generated.models import ComputePool as BaseComputePool
from snowflake.core.database import Database
from snowflake.core.database._generated.models.database import Database as BaseDatabase
from snowflake.core.image_repository import ImageRepository
from snowflake.core.image_repository._generated.models import ImageRepository as BaseImageRepository
from snowflake.core.managed_account import ManagedAccount
from snowflake.core.managed_account._generated.models import ManagedAccount as BaseManagedAccount
from snowflake.core.role import Role
from snowflake.core.role._generated.models import Role as BaseRole
from snowflake.core.schema import Schema
from snowflake.core.schema._generated.models import Schema as BaseSchema
from snowflake.core.task import Task
from snowflake.core.task._generated.models import Task as BaseTask
from snowflake.core.user import User
from snowflake.core.user._generated.models import User as BaseUser
from snowflake.core.warehouse import Warehouse
from snowflake.core.warehouse._generated.models import Warehouse as BaseWarehouse


@pytest.mark.parametrize(
    "base_obj, obj",
    [
        (
            BaseAccount(name="admin", edition="STANDARD", admin_name="admin", email="admin@localhost"),
            Account("admin", "STANDARD", "admin", "admin@localhost"),
        ),
        (
            BaseComputePool(name="cp", min_nodes=1, max_nodes=1, instance_family="fake"),
            ComputePool(name="cp", min_nodes=1, max_nodes=1, instance_family="fake"),
        ),
        (BaseDatabase(name="db"), Database("db")),
        (
            BaseManagedAccount(name="admin", admin_name="admin", admin_password="test123", account_type=""),
            ManagedAccount("admin", "admin", "test123", ""),
        ),
        (BaseRole(name="role"), Role("role")),
        (BaseImageRepository(name="rep"), ImageRepository("rep")),
        (BaseSchema(name="public"), Schema("public")),
        (BaseTask(name="task", definition="select 1"), Task("task", "select 1")),
        (BaseUser(name="user"), User("user")),
        (BaseWarehouse(name="wh"), Warehouse("wh")),
    ],
    ids=lambda t: type(t).__name__,
)
def test_repr(base_obj, obj):
    assert repr(base_obj) == repr(obj)



from io import BytesIO
from textwrap import dedent

import pytest as pytest

from snowflake.core.notebook import Notebook
from snowflake.core.stage import Stage
from tests.integ.utils import random_string


notebook_file = "test_notebook.ipynb"


@pytest.fixture(scope="module")
def table_name_for_execute_notebook_test():
    # This returns a table name that is used in the notebook to create a table;
    # We then check the existence of the table to verify the notebook execution
    return random_string(5, "test_table_execute_notebook")


@pytest.fixture(scope="module")
def notebook_stage_with_file(session, stages, table_name_for_execute_notebook_test) -> Stage:
    stage_name = random_string(5, "test_notebook_stage_")
    stage = Stage(name=stage_name, kind="PERMANENT")

    notebook_st = stages.create(stage)

    try:
        file_path = f"@{notebook_st.name}/{notebook_file}"

        session.file.put_stream(
            BytesIO(
                dedent(f"""{{ 
 "metadata": {{ 
  "kernelspec": {{ 
   "display_name": "Streamlit Notebook", 
   "name": "streamlit" 
  }} 
 }}, 
 "nbformat_minor": 5, 
 "nbformat": 4, 
 "cells": [ 
  {{ 
   "cell_type": "code", 
   "id": "8d50cbf4-0c8d-4950-86cb-114990437ac9", 
   "metadata": {{ 
    "language": "sql", 
    "name": "cell1" 
   }}, 
   "source": "create table {table_name_for_execute_notebook_test}(a int)", 
   "execution_count": null, 
   "outputs": [] 
  }}
 ] 
}}
""").encode()  # noqa
            ),
            file_path,
            auto_compress=False,
        )

        yield notebook_st
    finally:
        notebook_st.drop()


@pytest.fixture
def executable_notebook(set_internal_params, notebook_stage_with_file, notebooks, warehouse):
    notebook_name = random_string(5, "test_notebook_stage_")

    notebook = Notebook(
        name=notebook_name,
        from_location=f"@{notebook_stage_with_file.database.name}"
        f".{notebook_stage_with_file.schema.name}."
        f"{notebook_stage_with_file.name}",
        main_file=notebook_file,
        query_warehouse=warehouse.name,
    )

    nb = notebooks.create(notebook)

    with set_internal_params({"FEATURE_NOTEBOOKS_NON_INTERACTIVE_EXECUTION": "ENABLED"}):
        try:
            nb.add_live_version(from_last=True)
            yield nb
        finally:
            nb.drop()



from contextlib import suppress

import pytest

from pydantic import ValidationError

from tests.integ.table.conftest import assert_table
from tests.utils import random_string


def test_iter_like(tables, table_postfix, table_handle, table_handle_case_senstitive, database, schema):
    listed_tables_deep = list(tables.iter(like=table_handle.name, deep=True))
    assert_table(listed_tables_deep[0], table_handle.name, database, schema, True)

    listed_tables_not_deep = list(tables.iter(like=table_handle.name, deep=False))
    assert_table(listed_tables_not_deep[0], table_handle.name, database, schema, False)

    listed_tables_deep = list(tables.iter(like=f"test_table_{table_postfix}_case_sens%e", deep=True))
    assert_table(listed_tables_deep[0], table_handle_case_senstitive.name, database, schema, True)

    listed_tables_deep = list(tables.iter(like=f"test_table_{table_postfix}%", deep=True))
    assert len(listed_tables_deep) == 2


def test_iter_starts_with(tables, table_postfix, table_handle, table_handle_case_senstitive, database, schema):
    listed_tables_deep = list(tables.iter(starts_with=f"test_table_{table_postfix}_case_s", deep=True))
    assert len(listed_tables_deep) == 1
    assert_table(listed_tables_deep[0], table_handle_case_senstitive.name, database, schema, True)

    listed_tables_deep = list(tables.iter(starts_with=f"test_table_{table_postfix}_case_i".upper(), deep=True))
    assert len(listed_tables_deep) == 1
    assert_table(listed_tables_deep[0], table_handle.name, database, schema, True)


# The LIMIT keyword is required for the FROM keyword to function, limit=10 was chosen arbitrarily
# as it does not affect the test
def test_iter_from_name(tables, table_postfix, table_handle, table_handle_case_senstitive, database, schema):
    listed_tables_deep = list(tables.iter(limit=10, from_name="test_table", deep=True))
    assert len(listed_tables_deep) == 1
    assert_table(listed_tables_deep[0], table_handle_case_senstitive.name, database, schema, True)

    listed_tables_deep = list(tables.iter(limit=10, from_name=f"test_table_{table_postfix}_case_i".upper(), deep=True))
    assert 10 >= len(listed_tables_deep) >= 2
    assert_table(listed_tables_deep[0], table_handle.name, database, schema, True)


def test_iter_limit(tables):
    data = list(tables.iter(limit=10))
    assert len(data) <= 10

    with pytest.raises(ValidationError):
        data = list(tables.iter(limit=10001))


def test_iter_history(tables, table_handle):
    table_name = random_string(10, "test_table_")
    created_handle = tables.create(
        table_name, like_table=f"{table_handle.name}", copy_grants=True, mode="errorifexists"
    )
    try:
        assert created_handle.name == table_name
    finally:
        with suppress(Exception):
            created_handle.drop()

    data = list(tables.iter(history=True))
    assert len(data) >= 3



"""
Snowflake Notification Integration API.

The Snowflake Notification Integration API is a REST API that you can use to access, update, and perform certain actions on Notification Integration resource in a Snowflake database.  # noqa: E501

The version of the OpenAPI document: 0.0.1
Contact: support@snowflake.com
Generated by: https://openapi-generator.tech

Do not edit this file manually.
"""

from __future__ import annotations

import json
import pprint
import re  # noqa: F401

from typing import Any, Optional

from pydantic import ConfigDict, StrictStr

from snowflake.core.notification_integration._generated.models.notification_hook import (
    NotificationHook,
)


class NotificationQueueGcpPubsubInbound(NotificationHook):
    """A model object representing the NotificationQueueGcpPubsubInbound resource.

    Constructs an object of type NotificationQueueGcpPubsubInbound with the provided properties.

    Parameters
    __________
    gcp_pubsub_subscription_name : str, optional
        Pub/Sub topic subscription ID used to allow Snowflake access to event messages.
    gcp_pubsub_service_account : str, optional
        Google Cloud Platform (GCP) service account created for your account — **Read-only:** *any user-provided value will be ignored.*
    """

    gcp_pubsub_subscription_name: Optional[StrictStr] = None

    gcp_pubsub_service_account: Optional[StrictStr] = None

    __properties = ["type"]

    model_config = ConfigDict(
        validate_by_name=True,
        validate_assignment=True,
    )

    def to_str(self) -> str:
        """Returns the string representation of the model using alias."""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias."""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> NotificationQueueGcpPubsubInbound:
        """Create an instance of NotificationQueueGcpPubsubInbound from a JSON string."""
        return cls.from_dict(json.loads(json_str))

    def to_dict(
        self,
        hide_readonly_properties: bool = False,
    ) -> dict[str, Any]:
        """Returns the dictionary representation of the model using alias."""
        exclude_properties = set()

        if hide_readonly_properties:
            exclude_properties.update(
                {
                    "gcp_pubsub_service_account",
                }
            )

        _dict = self.model_dump(serialize_as_any=True, by_alias=True, exclude=exclude_properties, exclude_none=True)

        _dict["type"] = NotificationHook.get_child_model_discriminator_value("NotificationQueueGcpPubsubInbound")

        return _dict

    def to_dict_without_readonly_properties(self) -> dict[str, Any]:
        """Return the dictionary representation of the model without readonly properties."""
        return self.to_dict(hide_readonly_properties=True)

    @classmethod
    def from_dict(cls, obj: dict) -> NotificationQueueGcpPubsubInbound:
        """Create an instance of NotificationQueueGcpPubsubInbound from a dict."""
        if obj is None:
            return None

        if type(obj) is not dict:
            return NotificationQueueGcpPubsubInbound.model_validate(obj)

        _obj = NotificationQueueGcpPubsubInbound.model_validate(
            {
                "gcp_pubsub_subscription_name": obj.get("gcp_pubsub_subscription_name"),
                "gcp_pubsub_service_account": obj.get("gcp_pubsub_service_account"),
            }
        )

        return _obj


class NotificationQueueGcpPubsubInboundModel(NotificationHook):
    def __init__(
        self,  # optional properties
        gcp_pubsub_subscription_name: Optional[str] = None,
        gcp_pubsub_service_account: Optional[str] = None,
    ):
        """A model object representing the NotificationQueueGcpPubsubInbound resource.

        Constructs an object of type NotificationQueueGcpPubsubInbound with the provided properties.

        Parameters
        __________
        gcp_pubsub_subscription_name : str, optional
            Pub/Sub topic subscription ID used to allow Snowflake access to event messages.
        gcp_pubsub_service_account : str, optional
            Google Cloud Platform (GCP) service account created for your account.
        """
        super().__init__()
        self.gcp_pubsub_subscription_name = gcp_pubsub_subscription_name
        self.gcp_pubsub_service_account = gcp_pubsub_service_account

    __properties = ["type"]

    def __repr__(self) -> str:
        return repr(self._to_model())

    def _to_model(self):
        return NotificationQueueGcpPubsubInbound(
            gcp_pubsub_subscription_name=self.gcp_pubsub_subscription_name,
            gcp_pubsub_service_account=self.gcp_pubsub_service_account,
        )

    @classmethod
    def _from_model(cls, model) -> NotificationQueueGcpPubsubInboundModel:
        return NotificationQueueGcpPubsubInboundModel(
            gcp_pubsub_subscription_name=model.gcp_pubsub_subscription_name,
            gcp_pubsub_service_account=model.gcp_pubsub_service_account,
        )

    def to_dict(self):
        """Create a dictionary of the properties from a NotificationQueueGcpPubsubInbound.

        This method constructs a dictionary with the key-value entries corresponding to the properties of the NotificationQueueGcpPubsubInbound object.

        Returns
        _______
        dict
            A dictionary object created using the input model.
        """
        return self._to_model().to_dict()

    @classmethod
    def from_dict(cls, obj: dict) -> NotificationQueueGcpPubsubInboundModel:
        """Create an instance of NotificationQueueGcpPubsubInbound from a dict.

        This method constructs a NotificationQueueGcpPubsubInbound object from a dictionary with the key-value pairs of its properties.

        Parameters
        ----------
        obj : dict
            A dictionary whose keys and values correspond to the properties of the resource object.

        Returns
        _______
        NotificationQueueGcpPubsubInbound
            A NotificationQueueGcpPubsubInbound object created using the input dictionary; this will fail if the required properties are missing.
        """
        return cls._from_model(NotificationQueueGcpPubsubInbound.from_dict(obj))


NotificationQueueGcpPubsubInbound._model_class = NotificationQueueGcpPubsubInboundModel



from typing import TYPE_CHECKING, Optional

from snowflake.core import PollingOperation
from snowflake.core._common import CreateMode
from snowflake.core._internal.telemetry import api_telemetry

from ._generated.api.notebook_api_base import NotebookCollectionBase, NotebookResourceBase
from ._generated.models.notebook import Notebook
from ._generated.models.version_details import VersionDetails  # noqa


if TYPE_CHECKING:
    from snowflake.core.schema import SchemaResource


class NotebookCollection(NotebookCollectionBase):
    """Represents the collection operations of the Snowflake Notebook resource.

    With this collection, you can create, iterate through, and search for notebooks that you have access to
    in the current context.

    Examples
    ________
    Creating a notebook instance:

    >>> notebooks = root.databases["my_db"].schemas["my_schema"].notebooks
    >>> new_notebook = Notebook(name="my_notebook", comment="This is a notebook")
    >>> notebooks.create(new_notebook)
    """

    def __init__(self, schema: "SchemaResource"):
        super().__init__(schema, NotebookResource)

    @api_telemetry
    def create(  # type: ignore[override]
        self,
        notebook: Notebook,
        *,
        mode: CreateMode = CreateMode.error_if_exists,
    ) -> "NotebookResource":
        """Create a notebook in Snowflake.

        Parameters
        __________
        notebook: Notebook
            The ``Notebook`` object that you want to create in Snowflake.
        mode: CreateMode, optional
            One of the following strings.

            ``CreateMode.error_if_exists``: Throw an :class:`snowflake.core.exceptions.ConflictError`
            if the notebook already exists in Snowflake. Equivalent to SQL ``create notebook <name> ...``.

            ``CreateMode.or_replace``: Replace if the notebook already exists in Snowflake. Equivalent to SQL
            ``create or replace notebook <name> ...``.

            ``CreateMode.if_not_exists``: Do nothing if the notebook already exists in Snowflake. Equivalent to SQL
            ``create notebook <name> if not exists...``

            Default value is ``CreateMode.error_if_exists``.

        Examples
        ________
        Creating a notebook in Snowflake and getting the reference to it:

        >>> notebook = Notebook(name="my_notebook", version="notebook_ver1", comment="This is a notebook")
        >>> # Use the notebook collection created before to create a reference to the notebook resource
        >>> # in Snowflake.
        >>> notebook_reference = notebook_collection.create(notebook)
        """
        super().create(
            notebook=notebook,
            mode=mode,
        )
        return NotebookResource(notebook.name, self)

    @api_telemetry
    def create_async(  # type: ignore[override]
        self,
        notebook: Notebook,
        *,
        mode: CreateMode = CreateMode.error_if_exists,
    ) -> PollingOperation["NotebookResource"]:
        """An asynchronous version of :func:`create`.

        Refer to :class:`~snowflake.core.PollingOperation` for more information on asynchronous execution and
        the return type.
        """  # noqa: D401
        future = super().create_async(
            notebook=notebook,
            mode=mode,
        )
        return PollingOperation(future, lambda _: NotebookResource(notebook.name, self))


class NotebookResource(NotebookResourceBase):
    """Represents a reference to a Snowflake notebook.

    With this notebook reference, you can fetch information about a notebook, as well as perform
    certain actions on it.
    """

    _plural_name = "notebooks"

    @api_telemetry
    def rename(  # type: ignore[override]
        self,
        target_name: str,
        target_database: Optional[str] = None,
        target_schema: Optional[str] = None,
        if_exists: Optional[bool] = None,
    ) -> None:
        """Rename this notebook.

        Parameters
        __________
        target_name: str
            The new name of the notebook
        target_database: str, optional
            The new database name of the notebook. If not provided,
            the current database name is used. The default is ``None``.
        target_schema: str, optional
            The new schema name of the notebook. If not provided,
            the current schema name is used. The default is ``None``.
        if_exists: bool, optional
            Whether to check for the existence of notebook before
            renaming. The default is ``None``, which is equivalent to ``False``.

        Examples
        ________
        Renaming this notebook using its reference:

        >>> notebook_reference.rename("my_other_notebook")

        Renaming this notebook if it exists:

        >>> notebook_reference.rename("my_other_notebook", if_exists=True)
        """
        super().rename(
            target_database=target_database,
            target_schema=target_schema,
            target_name=target_name,
            if_exists=if_exists,
        )



import pytest

from snowflake.core import CreateMode
from snowflake.core.artifact_repository import ArtifactRepository
from snowflake.core.exceptions import ConflictError
from tests.integ.utils import random_string


def test_create_and_fetch_artifact_repository(artifact_repositories, temp_pypi_api_integration):
    """Test creating and fetching an artifact repository."""
    ar_name = random_string(10, "test_artifact_repository_")
    ar_def = ArtifactRepository(
        name=ar_name,
        type="PIP",
        api_integration=temp_pypi_api_integration,
        comment="created by test_create_and_fetch_artifact_repository",
    )
    ar = artifact_repositories.create(ar_def)
    try:
        fetched_ar = ar.fetch()
        assert fetched_ar.name == ar_name.upper()
        assert fetched_ar.type == ar_def.type
        assert fetched_ar.api_integration == temp_pypi_api_integration
        assert fetched_ar.comment == ar_def.comment
        assert fetched_ar.created_on is not None
        assert fetched_ar.database_name is not None
        assert fetched_ar.schema_name is not None
        assert fetched_ar.owner is not None
        assert fetched_ar.owner_role_type is not None
    finally:
        ar.drop()


def test_create_and_fetch_artifact_repository_minimal(artifact_repositories, temp_pypi_api_integration):
    """Test creating artifact repository with minimal required fields only."""
    ar_name = random_string(10, "test_artifact_repository_minimal_")
    ar_def = ArtifactRepository(
        name=ar_name,
        type="PIP",
        api_integration=temp_pypi_api_integration,
    )
    ar = artifact_repositories.create(ar_def)
    try:
        fetched_ar = ar.fetch()
        assert fetched_ar.name == ar_name.upper()
        assert fetched_ar.type == "PIP"
        assert fetched_ar.api_integration == temp_pypi_api_integration
        assert fetched_ar.comment is None
        assert fetched_ar.created_on is not None
        assert fetched_ar.database_name is not None
        assert fetched_ar.schema_name is not None
        assert fetched_ar.owner is not None
    finally:
        ar.drop()


def test_create_mode_error_if_exists(artifact_repositories, temp_pypi_api_integration):
    """Test CreateMode.error_if_exists - should fail when resource already exists."""
    ar_name = random_string(10, "test_artifact_repository_error_if_exists_")
    ar_def = ArtifactRepository(
        name=ar_name,
        type="PIP",
        api_integration=temp_pypi_api_integration,
        comment="test CreateMode.error_if_exists",
    )

    ar = artifact_repositories.create(ar_def)
    try:
        with pytest.raises(ConflictError):
            artifact_repositories.create(ar_def, mode=CreateMode.error_if_exists)
    finally:
        ar.drop()


def test_create_mode_or_replace(artifact_repositories, temp_pypi_api_integration):
    """Test CreateMode.or_replace - should replace existing resource."""
    ar_name = random_string(10, "test_artifact_repository_or_replace_")
    ar_def = ArtifactRepository(
        name=ar_name,
        type="PIP",
        api_integration=temp_pypi_api_integration,
        comment="original comment",
    )

    ar = artifact_repositories.create(ar_def)
    try:
        updated_ar_def = ArtifactRepository(
            name=ar_name,
            type="PIP",
            api_integration=temp_pypi_api_integration,
            comment="replaced comment",
        )
        artifact_repositories.create(updated_ar_def, mode=CreateMode.or_replace)

        fetched_ar = ar.fetch()
        assert fetched_ar.comment == "replaced comment"
    finally:
        ar.drop()


def test_create_mode_if_not_exists(artifact_repositories, temp_pypi_api_integration):
    """Test CreateMode.if_not_exists - should not error if resource exists and should not replace it."""
    ar_name = random_string(10, "test_artifact_repository_if_not_exists_")
    ar_def = ArtifactRepository(
        name=ar_name,
        type="PIP",
        api_integration=temp_pypi_api_integration,
        comment="original comment",
    )

    ar = artifact_repositories.create(ar_def, mode=CreateMode.if_not_exists)
    try:
        # Different comment to verify resource is not replaced
        different_ar_def = ArtifactRepository(
            name=ar_name,
            type="PIP",
            api_integration=temp_pypi_api_integration,
            comment="this comment should be ignored because resource already exists",
        )
        artifact_repositories.create(different_ar_def, mode=CreateMode.if_not_exists)

        fetched_ar = ar.fetch()
        assert fetched_ar.comment == "original comment"
    finally:
        ar.drop()



import pytest

from tests.utils import random_string

from .utils import create_service_function


pytestmark = [pytest.mark.skip_gov]


def test_drop_functions(temp_service_for_function, functions):
    funcs = []
    func_objects = []

    function_name_prefix = random_string(5, "test_func_")
    for i in range(5):
        function_name = f"{function_name_prefix}_foofunc_{str(i)}"
        function_name_with_args = f"{function_name}(REAL)"
        endpoint = "ep1"

        func_objects.append(
            create_service_function(
                function_name, ["REAL"], "REAL", endpoint, temp_service_for_function.name, functions
            )
        )

        funcs.append(function_name_with_args)

    ct = len([func.name for func in functions.iter()])

    for i in range(5):
        if i % 2 == 0:
            func_objects[i].drop()
        else:
            functions[funcs[i]].drop()
        functions["dummy_______()"].drop(if_exists=True)
        assert len([f.name for f in functions.iter()]) == ct - 1 - i



import importlib
import logging
import typing

from types import ModuleType

from snowflake.core.exceptions import MissingModuleError


logger = logging.getLogger(__name__)

snowpark_post_error_blurb = (
    "Please install it, as it is required for this functionality. For "
    "installation instructions see: https://developers.snowflake.com/snowpark/"
)


class MissingModule:
    """A class to replace missing modules.

    The only thing this class is supposed to do is raise a MissingModuleError when __getattr__ is called.
    This will be triggered whenever module.member is going to be called.
    """

    def __init__(self, mod_name: str, post_error_blurb: typing.Optional[str] = None) -> None:
        self.mod_name = mod_name
        self.post_error_blurb = post_error_blurb

    def __getattr__(self, item: str) -> typing.NoReturn:
        raise MissingModuleError(self.mod_name, self.post_error_blurb)


ModuleLikeObject = typing.Union[ModuleType, MissingModule]


def _import_or_missing_snowpark() -> tuple[ModuleLikeObject, bool]:
    try:
        snowpark = importlib.import_module("snowflake.snowpark")
        return snowpark, True
    except ImportError:
        return MissingModule("snowflake.snowpark", snowpark_post_error_blurb), False


(snowpark, installed_snowpark) = _import_or_missing_snowpark()

logger.info("checked whether snowflake-snowpark-python is available: %s", installed_snowpark)


def require_snowpark() -> None:
    """Raise an exception if snowflake-snowpark-python is missing."""
    if not installed_snowpark:
        raise MissingModuleError("snowflake.snowpark", snowpark_post_error_blurb)


__all__ = ["snowpark", "installed_snowpark", "require_snowpark"]



from typing import TYPE_CHECKING

from ._generated.api.network_rule_api_base import NetworkRuleCollectionBase, NetworkRuleResourceBase


if TYPE_CHECKING:
    from snowflake.core.schema import SchemaResource


class NetworkRuleCollection(NetworkRuleCollectionBase):
    """Represents the collection operations on the Snowflake network rule resource.

    With this collection, you can create, iterate through, and search for network rules that you have access to
    in the current context.
    """

    def __init__(self, schema: "SchemaResource"):
        super().__init__(schema, NetworkRuleResource)


class NetworkRuleResource(NetworkRuleResourceBase):
    """Represents a reference to a Snowflake network rule.

    With this network rule reference, you can fetch information about a network rule, as well as
    perform certain actions on it.
    """

    _plural_name = "network_rules"



"""
Snowflake Dynamic Table API.

The Snowflake Dynamic Table API is a REST API that you can use to access, update, and perform certain actions on Dynamic Tables resource in a Snowflake database.  # noqa: E501

The version of the OpenAPI document: 0.0.1
Contact: support@snowflake.com
Generated by: https://openapi-generator.tech

Do not edit this file manually.
"""

from __future__ import annotations

import json
import pprint
import re  # noqa: F401

from datetime import datetime
from typing import Any, List, Optional

from pydantic import BaseModel, ConfigDict, StrictBool, StrictInt, StrictStr, field_validator

from snowflake.core.dynamic_table._generated.models.dynamic_table_column import (
    DynamicTableColumn,
    DynamicTableColumnModel,
)
from snowflake.core.dynamic_table._generated.models.target_lag import TargetLag, TargetLagModel


class DynamicTable(BaseModel):
    """A model object representing the DynamicTable resource.

    Constructs an object of type DynamicTable with the provided properties.

    Parameters
    __________
    name : str
        Specifies the name for the dynamic table, must be unique for the schema in which the dynamic table is created
    target_lag : TargetLag

    warehouse : str
        Specifies the name of the warehouse that provides the compute resources for refreshing the dynamic table
    query : str
        Specifies the query whose results the dynamic table should contain
    kind : str,  default 'PERMANENT'
        Specifies the dynamic table type, permanent (default) or transient.
    columns : list[DynamicTableColumn], optional

    refresh_mode : str, optional
        Specifies the refresh type for the dynamic table
    initialize : str, optional
        Specifies the behavior of the initial refresh of the dynamic table
    cluster_by : list[str], optional
        Specifies one or more columns or column expressions in the dynamic table as the clustering key
    data_retention_time_in_days : int, optional
        Specifies the retention period for the dynamic table so that Time Travel actions (SELECT, CLONE) can be performed on historical data in the dynamic table
    max_data_extension_time_in_days : int, optional
        Specifies the retention period for the dynamic table so that Time Travel actions (SELECT, CLONE) can be performed on historical data in the dynamic table
    comment : str, optional
        Specifies a comment for the dynamic table.
    created_on : datetime, optional
        Date and time when the dynamic table was created — **Read-only:** *any user-provided value will be ignored.*
    database_name : str, optional
        Database in which the dynamic table is stored — **Read-only:** *any user-provided value will be ignored.*
    schema_name : str, optional
        Schema in which the dynamic table is stored — **Read-only:** *any user-provided value will be ignored.*
    rows : int, optional
        Number of rows in the dynamic table — **Read-only:** *any user-provided value will be ignored.*
    bytes : int, optional
        Number of bytes that will be scanned if the entire table is scanned in a query. Note that this number may be different than the number of actual physical bytes stored on-disk for the table — **Read-only:** *any user-provided value will be ignored.*
    scheduling_state : str, optional
        Scheduling state (RUNNING or SUSPENDED) — **Read-only:** *any user-provided value will be ignored.*
    automatic_clustering : bool, optional
        If Automatic Clustering is enabled for your account, specifies whether it is explicitly enabled or disabled for the dynamic table — **Read-only:** *any user-provided value will be ignored.*
    owner : str, optional
        Role that owns the table — **Read-only:** *any user-provided value will be ignored.*
    owner_role_type : str, optional
        The type of role that owns the object — **Read-only:** *any user-provided value will be ignored.*
    budget : str, optional
        Name of the budget if the object is monitored by a budget — **Read-only:** *any user-provided value will be ignored.*
    """

    name: StrictStr

    kind: Optional[StrictStr] = "PERMANENT"

    columns: Optional[List[DynamicTableColumn]] = None

    target_lag: TargetLag

    refresh_mode: Optional[StrictStr] = None

    initialize: Optional[StrictStr] = None

    warehouse: StrictStr

    cluster_by: Optional[List[StrictStr]] = None

    query: StrictStr

    data_retention_time_in_days: Optional[StrictInt] = None

    max_data_extension_time_in_days: Optional[StrictInt] = None

    comment: Optional[StrictStr] = None

    created_on: Optional[datetime] = None

    database_name: Optional[StrictStr] = None

    schema_name: Optional[StrictStr] = None

    rows: Optional[StrictInt] = None

    bytes: Optional[StrictInt] = None

    scheduling_state: Optional[StrictStr] = None

    automatic_clustering: Optional[StrictBool] = None

    owner: Optional[StrictStr] = None

    owner_role_type: Optional[StrictStr] = None

    budget: Optional[StrictStr] = None

    __properties = [
        "name",
        "kind",
        "columns",
        "target_lag",
        "refresh_mode",
        "initialize",
        "warehouse",
        "cluster_by",
        "query",
        "data_retention_time_in_days",
        "max_data_extension_time_in_days",
        "comment",
        "created_on",
        "database_name",
        "schema_name",
        "rows",
        "bytes",
        "scheduling_state",
        "automatic_clustering",
        "owner",
        "owner_role_type",
        "budget",
    ]

    @field_validator("kind")
    def kind_validate_enum(cls, v):
        if v is None:
            return v
        if v not in ("PERMANENT", "TRANSIENT"):
            raise ValueError("must validate the enum values ('PERMANENT','TRANSIENT')")
        return v

    @field_validator("refresh_mode")
    def refresh_mode_validate_enum(cls, v):
        if v is None:
            return v
        if v not in ("AUTO", "FULL", "INCREMENTAL"):
            raise ValueError("must validate the enum values ('AUTO','FULL','INCREMENTAL')")
        return v

    @field_validator("initialize")
    def initialize_validate_enum(cls, v):
        if v is None:
            return v
        if v not in ("ON_CREATE", "ON_SCHEDULE"):
            raise ValueError("must validate the enum values ('ON_CREATE','ON_SCHEDULE')")
        return v

    @field_validator("scheduling_state")
    def scheduling_state_validate_enum(cls, v):
        if v is None:
            return v
        if v not in ("RUNNING", "SUSPENDED"):
            raise ValueError("must validate the enum values ('RUNNING','SUSPENDED')")
        return v

    model_config = ConfigDict(
        validate_by_name=True,
        validate_assignment=True,
    )

    def to_str(self) -> str:
        """Returns the string representation of the model using alias."""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias."""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> DynamicTable:
        """Create an instance of DynamicTable from a JSON string."""
        return cls.from_dict(json.loads(json_str))

    def to_dict(
        self,
        hide_readonly_properties: bool = False,
    ) -> dict[str, Any]:
        """Returns the dictionary representation of the model using alias."""
        exclude_properties = set()

        if hide_readonly_properties:
            exclude_properties.update(
                {
                    "created_on",
                    "database_name",
                    "schema_name",
                    "rows",
                    "bytes",
                    "scheduling_state",
                    "automatic_clustering",
                    "owner",
                    "owner_role_type",
                    "budget",
                }
            )

        _dict = self.model_dump(serialize_as_any=True, by_alias=True, exclude=exclude_properties, exclude_none=True)

        # override the default output from pydantic by calling `to_dict()` of each item in columns (list)
        _items = []
        if self.columns:
            for _item in self.columns:
                if _item:
                    _items.append(_item.to_dict())
            _dict["columns"] = _items

        # override the default output from pydantic by calling `to_dict()` of target_lag
        if self.target_lag:
            _dict["target_lag"] = self.target_lag.to_dict()

        return _dict

    def to_dict_without_readonly_properties(self) -> dict[str, Any]:
        """Return the dictionary representation of the model without readonly properties."""
        return self.to_dict(hide_readonly_properties=True)

    @classmethod
    def from_dict(cls, obj: dict) -> DynamicTable:
        """Create an instance of DynamicTable from a dict."""
        if obj is None:
            return None

        if type(obj) is not dict:
            return DynamicTable.model_validate(obj)

        _obj = DynamicTable.model_validate(
            {
                "name": obj.get("name"),
                "kind": obj.get("kind") if obj.get("kind") is not None else "PERMANENT",
                "columns": [DynamicTableColumn.from_dict(_item) for _item in obj.get("columns")]
                if obj.get("columns") is not None
                else None,
                "target_lag": TargetLag.from_dict(obj.get("target_lag")) if obj.get("target_lag") is not None else None,
                "refresh_mode": obj.get("refresh_mode"),
                "initialize": obj.get("initialize"),
                "warehouse": obj.get("warehouse"),
                "cluster_by": obj.get("cluster_by"),
                "query": obj.get("query"),
                "data_retention_time_in_days": obj.get("data_retention_time_in_days"),
                "max_data_extension_time_in_days": obj.get("max_data_extension_time_in_days"),
                "comment": obj.get("comment"),
                "created_on": obj.get("created_on"),
                "database_name": obj.get("database_name"),
                "schema_name": obj.get("schema_name"),
                "rows": obj.get("rows"),
                "bytes": obj.get("bytes"),
                "scheduling_state": obj.get("scheduling_state"),
                "automatic_clustering": obj.get("automatic_clustering"),
                "owner": obj.get("owner"),
                "owner_role_type": obj.get("owner_role_type"),
                "budget": obj.get("budget"),
            }
        )

        return _obj


class DynamicTableModel:
    def __init__(
        self,
        name: str,
        target_lag: TargetLag,
        warehouse: str,
        query: str,
        # optional properties
        kind: Optional[str] = "PERMANENT",
        columns: Optional[list[DynamicTableColumn]] = None,
        refresh_mode: Optional[str] = None,
        initialize: Optional[str] = None,
        cluster_by: Optional[list[str]] = None,
        data_retention_time_in_days: Optional[int] = None,
        max_data_extension_time_in_days: Optional[int] = None,
        comment: Optional[str] = None,
        created_on: Optional[datetime] = None,
        database_name: Optional[str] = None,
        schema_name: Optional[str] = None,
        rows: Optional[int] = None,
        bytes: Optional[int] = None,
        scheduling_state: Optional[str] = None,
        automatic_clustering: Optional[bool] = None,
        owner: Optional[str] = None,
        owner_role_type: Optional[str] = None,
        budget: Optional[str] = None,
    ):
        """A model object representing the DynamicTable resource.

        Constructs an object of type DynamicTable with the provided properties.

        Parameters
        __________
        name : str
            Specifies the name for the dynamic table, must be unique for the schema in which the dynamic table is created
        target_lag : TargetLag

        warehouse : str
            Specifies the name of the warehouse that provides the compute resources for refreshing the dynamic table
        query : str
            Specifies the query whose results the dynamic table should contain
        kind : str,  default 'PERMANENT'
            Specifies the dynamic table type, permanent (default) or transient.
        columns : list[DynamicTableColumn], optional

        refresh_mode : str, optional
            Specifies the refresh type for the dynamic table
        initialize : str, optional
            Specifies the behavior of the initial refresh of the dynamic table
        cluster_by : list[str], optional
            Specifies one or more columns or column expressions in the dynamic table as the clustering key
        data_retention_time_in_days : int, optional
            Specifies the retention period for the dynamic table so that Time Travel actions (SELECT, CLONE) can be performed on historical data in the dynamic table
        max_data_extension_time_in_days : int, optional
            Specifies the retention period for the dynamic table so that Time Travel actions (SELECT, CLONE) can be performed on historical data in the dynamic table
        comment : str, optional
            Specifies a comment for the dynamic table.
        created_on : datetime, optional
            Date and time when the dynamic table was created.
        database_name : str, optional
            Database in which the dynamic table is stored
        schema_name : str, optional
            Schema in which the dynamic table is stored
        rows : int, optional
            Number of rows in the dynamic table.
        bytes : int, optional
            Number of bytes that will be scanned if the entire table is scanned in a query. Note that this number may be different than the number of actual physical bytes stored on-disk for the table
        scheduling_state : str, optional
            Scheduling state (RUNNING or SUSPENDED)
        automatic_clustering : bool, optional
            If Automatic Clustering is enabled for your account, specifies whether it is explicitly enabled or disabled for the dynamic table.
        owner : str, optional
            Role that owns the table
        owner_role_type : str, optional
            The type of role that owns the object.
        budget : str, optional
            Name of the budget if the object is monitored by a budget
        """
        self.name = name
        self.kind = kind
        self.columns = columns
        self.target_lag = target_lag
        self.refresh_mode = refresh_mode
        self.initialize = initialize
        self.warehouse = warehouse
        self.cluster_by = cluster_by
        self.query = query
        self.data_retention_time_in_days = data_retention_time_in_days
        self.max_data_extension_time_in_days = max_data_extension_time_in_days
        self.comment = comment
        self.created_on = created_on
        self.database_name = database_name
        self.schema_name = schema_name
        self.rows = rows
        self.bytes = bytes
        self.scheduling_state = scheduling_state
        self.automatic_clustering = automatic_clustering
        self.owner = owner
        self.owner_role_type = owner_role_type
        self.budget = budget

    __properties = [
        "name",
        "kind",
        "columns",
        "target_lag",
        "refresh_mode",
        "initialize",
        "warehouse",
        "cluster_by",
        "query",
        "data_retention_time_in_days",
        "max_data_extension_time_in_days",
        "comment",
        "created_on",
        "database_name",
        "schema_name",
        "rows",
        "bytes",
        "scheduling_state",
        "automatic_clustering",
        "owner",
        "owner_role_type",
        "budget",
    ]

    def __repr__(self) -> str:
        return repr(self._to_model())

    def _to_model(self):
        return DynamicTable(
            name=self.name,
            kind=self.kind,
            columns=[x._to_model() for x in self.columns] if self.columns is not None else None,
            target_lag=self.target_lag._to_model() if self.target_lag is not None else None,
            refresh_mode=self.refresh_mode,
            initialize=self.initialize,
            warehouse=self.warehouse,
            cluster_by=self.cluster_by,
            query=self.query,
            data_retention_time_in_days=self.data_retention_time_in_days,
            max_data_extension_time_in_days=self.max_data_extension_time_in_days,
            comment=self.comment,
            created_on=self.created_on,
            database_name=self.database_name,
            schema_name=self.schema_name,
            rows=self.rows,
            bytes=self.bytes,
            scheduling_state=self.scheduling_state,
            automatic_clustering=self.automatic_clustering,
            owner=self.owner,
            owner_role_type=self.owner_role_type,
            budget=self.budget,
        )

    @classmethod
    def _from_model(cls, model) -> DynamicTableModel:
        return DynamicTableModel(
            name=model.name,
            kind=model.kind,
            columns=[DynamicTableColumnModel._from_model(x) for x in model.columns] if model.columns else None,
            target_lag=TargetLagModel._from_model(model.target_lag) if model.target_lag else None,
            refresh_mode=model.refresh_mode,
            initialize=model.initialize,
            warehouse=model.warehouse,
            cluster_by=model.cluster_by,
            query=model.query,
            data_retention_time_in_days=model.data_retention_time_in_days,
            max_data_extension_time_in_days=model.max_data_extension_time_in_days,
            comment=model.comment,
            created_on=model.created_on,
            database_name=model.database_name,
            schema_name=model.schema_name,
            rows=model.rows,
            bytes=model.bytes,
            scheduling_state=model.scheduling_state,
            automatic_clustering=model.automatic_clustering,
            owner=model.owner,
            owner_role_type=model.owner_role_type,
            budget=model.budget,
        )

    def to_dict(self):
        """Create a dictionary of the properties from a DynamicTable.

        This method constructs a dictionary with the key-value entries corresponding to the properties of the DynamicTable object.

        Returns
        _______
        dict
            A dictionary object created using the input model.
        """
        return self._to_model().to_dict()

    @classmethod
    def from_dict(cls, obj: dict) -> DynamicTableModel:
        """Create an instance of DynamicTable from a dict.

        This method constructs a DynamicTable object from a dictionary with the key-value pairs of its properties.

        Parameters
        ----------
        obj : dict
            A dictionary whose keys and values correspond to the properties of the resource object.

        Returns
        _______
        DynamicTable
            A DynamicTable object created using the input dictionary; this will fail if the required properties are missing.
        """
        return cls._from_model(DynamicTable.from_dict(obj))


DynamicTable._model_class = DynamicTableModel
